from sherlockpipe.star.StarCatalog import StarCatalog
from astroquery.vizier import Vizier
import transitleastsquares as tls


class KicStarCatalog(StarCatalog):
    def __init__(self):
        super().__init__()

    def catalog_info(self, id):
        """Takes KIC_ID, returns stellar information from online catalog using Vizier"""
        if type(id) is not int:
            raise TypeError('KIC_ID ID must be of type "int"')
        columns = ["Teff", "log(g)", "Rad", "E_Rad", "e_Rad", "Mass", "E_Mass", "e_Mass", "_RA", "_DE"]
        catalog = "J/ApJS/229/30/catalog"
        result = (
            Vizier(columns=columns)
                .query_constraints(KIC=id, catalog=catalog)[0]
                .as_array()
        )
        Teff = result[0][0]
        logg = result[0][1]
        radius = result[0][2]
        radius_max = result[0][3]
        radius_min = result[0][4]
        mass = result[0][5]
        mass_max = result[0][6]
        mass_min = result[0][7]
        ra = result[0][8]
        dec = result[0][9]
        lum = self.star_luminosity(Teff, radius)
        ld, mass, mass_min, mass_max, radius, radius_min, radius_max = tls.catalog_info(KIC_ID=id)
        return (ld, Teff, lum, logg, radius, radius_min, radius_max, mass, mass_min, mass_max, ra, dec)



import os 
import numpy as np
import pandas as pd

from os.path import exists
from reportlab.lib import colors
from reportlab.lib.units import mm
from reportlab.pdfgen import canvas
from reportlab.lib.units import inch

# modules to generate the a pdf validation report
from reportlab.lib.pagesizes import letter, A4
from reportlab.pdfbase.pdfmetrics import stringWidth
from reportlab.lib.colors import red, black, grey, darkcyan
from reportlab.lib.enums import TA_RIGHT, TA_JUSTIFY, TA_CENTER
from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle
from reportlab.platypus import SimpleDocTemplate, Paragraph, Spacer, Image,  Table, TableStyle, PageBreak, Flowable


def LATTE_DV(tic, indir, syspath, transit_list, sectors_all, target_ra, target_dec, tessmag, teff, srad, bls_stats1, bls_stats2, tpf_corrupt, astroquery_corrupt, FFI, bls = False, model = False, mpi = False, test = 'no'):

	'''
	funtion that makes compiles all of the information and figures into a comprehensive pdf summary document.

    Parameters
    ----------
    tic  :   str
        target TIC ID
    indir  :  str
        path to directory where all the plots and data will be saved. 
    sectors_all  :   list
        all the sectors in which the target has been/ will be observed
    target_ra   :  float
        the right ascension of the target
    target_dec   :  float
        the declination of the target
    tessmag  :  float
        TESS magnitude of the target star
    teff  :  float
        effective temperature of the tagret star (K)
    srad  :  float
        radius of the target star (solar radii)
	bls_stats1  :  list
		list of the returned stats of the initial bls search
	bls_stats2  :  list
		list of the returned stats of the second bls search
	FFI  :  bool
		whether the input data is from FFIs (True = from FFIs)
	bls  :  bool  (false)
		whether the bls search was run 
	model  :  bool  (false)
		whether the transit was modelled (only works if payenti has sucessfully been installed)
    Returns
    -------
    LATTE Data Validation report in PDF format.

	'''

	# ---- CHECK WHETHER THE TARGET IS A TCE OR A TOI ----
	print ("\n Start compiling the data validation report...")

	# TCE -----
	lc_dv = np.genfromtxt('{}/data/tesscurl_sector_all_dv.sh'.format(indir), dtype = str)

	TCE_links = []

	for i in lc_dv:
		if str(tic) in str(i[6]):
			TCE_links.append(i[6])

	if len(TCE_links) == 0:
		TCE = " - "
	
	else:
		TCE_links = np.sort(TCE_links)
		TCE_link = TCE_links[0]  # this link should allow you to acess the MAST DV report
		TCE = 'Yes **'
		TCE_link = '<link href="%s" color="blue">here</link>' % TCE_link
	

	# TOI -----
	TOI_planets = pd.read_csv('{}/data/TOI_list.txt'.format(indir), comment = "#")
	
	TOIpl = TOI_planets.loc[TOI_planets['TIC'] == float(tic)]
		
	if len(TOIpl) == 0:
		TOI = ' -  '
	else:
		TOI = (float(TOIpl["Full TOI ID"]))
	
	# ------ PARAMS ------
	ra = float(target_ra)
	dec = float(target_dec)
	
	
	def addPageNumber(canvas, doc):
		"""
		Add the page numbers to the document
		"""
		width, height = A4 # this is useful when defining where to plot something on the page
	
		page_num = canvas.getPageNumber()
		text = "%s" % page_num
		header = "TIC {}".format(tic)
		
		canvas.setFont('Helvetica',8)
		canvas.drawString(width*0.85, height * 0.95, header)
		canvas.drawRightString(200*mm, 10*mm, text)
 

	#------------------------------------------------------------------
	# Recall the names of all the plots that will be in the DV report
	#------------------------------------------------------------------
	
	# plot the full light curve, with marked sectors and marked transit - binned and unbinned

	full_LC_name = "{}/{}/{}_fullLC_md.png".format(indir,tic,tic)
	
	background_flux_name = '{}/{}/{}_background.png'.format(indir, tic, tic)
	
	centroid_positions_name = '{}/{}/{}_centroids.png'.format(indir, tic, tic)
	
	flux_aperture_name= '{}/{}/{}_aperture_size.png'.format(indir, tic, tic)
	
	tess_stars_name = '{}/{}/{}_star_field.png'.format(indir, tic, tic)
	
	#SDSS_stars_name = '{}/{}/{}_SDSSstar_field.png'.format(indir, tic, tic)

	nearest_neighbour_name = '{}/{}/{}_nearest_neighbours.png'.format(indir, tic, tic)
	
	pixel_LCs_name = '{}/{}/{}_individual_pixel_LCs_0.png'.format(indir, tic,tic)
	
	bls1 = '{}/{}/{}_bls_first.png'.format(indir, tic, tic)
	
	bls2 = '{}/{}/{}_bls_second.png'.format(indir, tic, tic)
	
	in_out_name = '{}/{}/{}_flux_comparison.png'.format(indir, tic, tic)
	
	model_name = '{}/{}/model_out/{}b_tr.png'.format(indir, tic, tic)

	phasefold_name = '{}/{}/{}_phase_folded.png'.format(indir, tic, tic)
	
	apertures_name = '{}/{}/{}_apertures_0.png'.format(indir, tic, tic)
	
	# ----- LOGOS ------
	# if this is a unittest run, find the files for the logos stored in the test folder
	if test != 'no':
		PHT_logo_name  =  '{}/LATTE_imgs/PHT_logo.jpg'.format(test)
		LATTE_logo_name = '{}/LATTE_imgs/LATTE_logo.png'.format(test)
		TESS_logo_name  = '{}/LATTE_imgs/TESS_logo.png'.format(test)

	# otherwise they're located in the place where the program is insatlled. 
	else:
		PHT_logo_name  =  '{}/LATTE_imgs/PHT_logo.jpg'.format(syspath)
		LATTE_logo_name = '{}/LATTE_imgs/LATTE_logo.png'.format(syspath)
		TESS_logo_name  = '{}/LATTE_imgs/TESS_logo.png'.format(syspath)


	# -------------------------------------------
	# Make a PDF summary file
	# -------------------------------------------

	doc = SimpleDocTemplate("{}/{}/DV_report_{}.pdf".format(indir,tic,tic) ,pagesize=A4,
						   rightMargin=72,leftMargin=72,
						   topMargin=40,bottomMargin=20)

	width, height = A4 # this is useful when defining where to plot something on the page
	
	Story=[]

	fig_count = 0
	table_count = 0
	
	# title
	title = "PHT Data Validation Report"
	subheading = "TIC {}".format(tic)

	styles=getSampleStyleSheet()
	styles.add(ParagraphStyle(name='centre', alignment=TA_CENTER))
	ptext = '<font size=12><b>%s</b></font>' % title
	subtext = '<font size=12><b>%s</b></font>' % subheading
	
	Story.append(Paragraph(ptext, styles["centre"]))
	Story.append(Paragraph(subtext, styles["centre"]))
	Story.append(Spacer(1, 30))
	
	# ----- ADD THE LOGOS -------
	PHT_logo =   Image(PHT_logo_name)
	PHT_logo.drawHeight = 0.5*inch*PHT_logo.drawHeight / PHT_logo.drawWidth
	PHT_logo.drawWidth = 0.5*inch

	LATTE_logo = Image(LATTE_logo_name)
	LATTE_logo.drawHeight = 0.5*inch*LATTE_logo.drawHeight / LATTE_logo.drawWidth
	LATTE_logo.drawWidth = 0.5*inch

	TESS_logo =  Image(TESS_logo_name)
	TESS_logo.drawHeight = 0.8*inch*TESS_logo.drawHeight / TESS_logo.drawWidth
	TESS_logo.drawWidth = 0.8*inch


	logo_table = (Table([[PHT_logo, LATTE_logo, TESS_logo]],
					colWidths=[width * 0.1],
					rowHeights=[1 * mm]))

	logo_table.setStyle(TableStyle([('ALIGN', (0, 0), (-1, -1), 'CENTRE'),('VALIGN', (0, 0), (-1,-1), 'MIDDLE')]))
	
	Story.append(logo_table)

	Story.append(Spacer(1, 30))
	# ----------------------------

	line = MCLine(width*0.77)
	Story.append(line)
	Story.append(Spacer(1, 2))
	line = MCLine_color(width*0.77)
	Story.append(line)
	Story.append(Spacer(1, 20))

	# --------------------------------------------
	# Full Image with momentum dumps
	# --------------------------------------------
	im = Image(full_LC_name)
	im._restrictSize(width*0.8, width*0.8)
	Story.append(im)
	
	fig_count += 1

	full_image_text = "Fig {}. Full lightcurve for target TIC {}. The solid red lines at the bottom of the figure indicated the \
	times of the reaction wheel momentum dumps and the dashed black line(s) show the time(s) of the marked transit event(s). Momentum dumps \
	occur around every 2 to 2.5 days and typically last around half an hour.".format(fig_count,tic)
	
	ptext = '<font size=8>%s</font>' % full_image_text
	Story.append(Paragraph(ptext, styles["Normal"]))
	
	# --------------------------------------
	# ------ stellar parameters table ------
	# --------------------------------------
	try:
		srad = float(srad)
	except:
		srad = -999
	
	try:
		teff = float(teff)
	except:
		teff = -999
			
	data_stellar= [['Parameter',  "Value", "Unit"],
				   ['TIC ID',	 tic, ],
				   ['RA',		 ra, "degrees"],
				   ['Dec',		dec, "degrees"],
				   ['Radius',	 "{:.4f}".format(srad), "Solar Radii"],
				   ['Tess Mag',   "{:.4f}".format(tessmag), "Mag"],
				   ['Teff',	   "{:.0f}".format(teff), "Kelvin"],
				   ['Sectors',	  "{} *".format(str(sectors_all)[1:-1]), ],
				   ['TCE',	  TCE, ],
				   ['TOI',	  "{}".format(str(TOI)), ],
				   ]
	
	table_stellar=Table(data_stellar)
	table_stellar=Table(data_stellar,colWidths=width * 0.2, style=[
						('LINEABOVE',(0,1),(-1,1),1,colors.black),
						('LINEABOVE',(0,10),(-1,10),1,colors.black),
						('FONTSIZE', (0,0),(-1,9), 8),
						])

	data_len = len(data_stellar)
	
	for each in range(data_len):
		if each % 2 == 0:
			bg_color = colors.whitesmoke
		else:
			bg_color = colors.white
	
		table_stellar.setStyle(TableStyle([('BACKGROUND', (0, each), (-1, each), bg_color)]))
	

	# ------ ADD A LINE TO SEPERATE SECTIONS -----

	Story.append(Spacer(1, 20))
	line = MCLine(width*0.77)
	Story.append(line)

	# ------
	
	Story.append(Spacer(1, 20))
	ptext = '<font size=9>Target Properties</font>'
	Story.append(Paragraph(ptext, styles["Normal"]))
	Story.append(Spacer(1, 12))
	
	Story.append(table_stellar)
	Story.append(Spacer(1, 15))
	
	table_count += 1

	exofop_url = "https://exofop.ipac.caltech.edu/tess/target.php?id={}".format(tic)
	exofop_link = '<link href="%s" color="blue">TIC %s</link>' % (exofop_url, tic)


	if TCE == 'Yes **':
		Stellartable_text = "Table {}. Stellar properties of {}. \
			* List of the sectors in which the target will be has been \
			observed. ** Click {} for the TCE report.".format(table_count, exofop_link, TCE_link)


	else:
		Stellartable_text = "Table {}. Stellar properties of the {}. \
			* List of the sectors in which the target will be has been \
			observed.".format(table_count,exofop_link)

		


	ptext = '<font size=8>%s</font>' % Stellartable_text
	Story.append(Paragraph(ptext, styles["Normal"]))


	# --------------------------------------------
	# Background
	# --------------------------------------------
	Story.append(PageBreak()) # always start a new page for this analysis
	im2 = Image(background_flux_name)
	
	if len(transit_list) == 1:
		im2._restrictSize(width*0.55, width*0.55)
	
	else:
		im2._restrictSize(width*0.8, width*0.8)
	
	Story.append(im2)

	fig_count += 1
	Story.append(Spacer(1, 1))
	background_text = "Fig {}. Background flux vs. time around the time of each transit-like event. \
		The vertical orange line indicates the time of the transit-like event.".format(fig_count)
	
	ptext = '<font size=8>%s</font>' % background_text
	Story.append(Paragraph(ptext, styles["Normal"]))
	

	# --------------------------------------------
	# Centroid Position
	# --------------------------------------------
	
	if FFI == False:
		Story.append(Spacer(1, 10))
		im3 = Image(centroid_positions_name)

		if len(transit_list) == 1:
			im3._restrictSize(width*0.52, width*0.52)
		else:
			im3._restrictSize(width*0.7, width*0.7)
	
		Story.append(im3)
		
		fig_count += 1
		centroid_text = "Fig {}. The x and y centroid positions around the time of each transit-like event. The black points shows the CCD column and row position of the target’s flux-weighted centroid. \
			The red shows the CCD column and row local motion differential velocity aberration (DVA), pointing drift, and thermal effects. \
			The vertical orange line indicates the time of the transit-like event".format(fig_count)
		
		ptext = '<font size=8>%s</font>' % centroid_text

		Story.append(Spacer(1, 5))

		Story.append(Paragraph(ptext, styles["Normal"]))
		
		Story.append(Spacer(1, 13))

		#Story.append(PageBreak()) # always start a new page for this analysis

	# the following plots will only exist if the TPF file is not corrupt - otherwise skip these.
	if tpf_corrupt == False:

		# --------------------------------------------
		# Flux Aperture
		# --------------------------------------------
		im4 = Image(flux_aperture_name)
	
		if len(transit_list) == 1:
			im4._restrictSize(width*0.55, width*0.55)
		else:
			im4._restrictSize(width*0.7, width*0.7)
		Story.append(im4)
		
		fig_count += 1
		Story.append(Spacer(1, 10))
		flux_aperture_text = "Fig {}. The lightcurve around the time of each transit-like event extracted with the SPOC pipeline \
			defined aperture (binned:blue, unbinned:grey) and the with an aperture that is 40 per cent smaller (red). The flux is extracted \
			from the target pixel files (TPFs) and has not been detrended or \
			corrected for systematics. The vertical orange line indicates the time of the transit-like event.".format(fig_count)
		
		ptext = '<font size=8>%s</font>' % flux_aperture_text
		Story.append(Paragraph(ptext, styles["Normal"]))
		
		# --------------------------------------------
		# Apertures Sizes
		# --------------------------------------------
		im45 = Image(apertures_name)
		
		im45._restrictSize(width*0.4, width*0.4)
		
		Story.append(im45)
		
		fig_count += 1

		Story.append(Spacer(1, 10))

		if FFI == False:
			aperture_text = "Fig {}. The apertures used to extract the lightcurves. The blue aperture on the right shows the \
			optimum aperture determined by the SPOC pipeline, which is used for the extraction of 2-minute cadence light curves shown in Figure 1. \
			The red outline on the left shows an aperture that is around 40 per cent smaller than the SPOC pipeline aperture which was used to extract the \
			red lightcurve shown in Figure {}.".format(fig_count, (fig_count-1))
		else:
			aperture_text = "Fig {}. The larger (right hand side, blue) and the smaller (left hamd side, red) apertures used to extract the lightcurves shown in Figure {}.".format(fig_count, (fig_count-1))

		ptext = '<font size=8>%s</font>' % aperture_text
		Story.append(Paragraph(ptext, styles["Normal"]))
			
		
		# --------------------------------------------
		# In and Out of Transit Comparison
		# --------------------------------------------
		
		Story.append(Spacer(1, 12))
		im5 = Image(in_out_name)
	
	
		im5._restrictSize(width*0.9, width*0.9)
	
		Story.append(im5)
	
		fig_count += 1
		Story.append(Spacer(1, 10))
		flux_aperture_text = "Fig {}. Difference images for target TIC {} for each transit like event. \
		Left: mean in-transit flux(left). Middle: mean out-of-transit flux. Right: difference between the mean out-of-transit and mean in-transit flux. \
		Ensure that the change in brightness occurs on target.".format(fig_count, tic)
		
		ptext = '<font size=8>%s</font>' % flux_aperture_text
		Story.append(Paragraph(ptext, styles["Normal"]))
	
	
		# --------------------------------------------
		# tess stars + SDSS star field
		# --------------------------------------------
		# can only put this in the report if astroquery is working. 
		if astroquery_corrupt == False:

			Story.append(Spacer(1, 12))
		
			im6 = Image(tess_stars_name)
			
			# if not with mpi (two star images)
			if mpi == False:
				im6._restrictSize(width*0.7, width*0.5)
			else:
				im6._restrictSize(width*0.35, width*0.35)
	
			Story.append(im6)
		
			fig_count += 1
			tess_stars_text = "Fig {}. Left: The locations of nearby GAIA DR2 stars with mag < 15 (orange circle) within the Tess \
			Cut Out around TIC {} (red star). Only shown for one sector. Right: SDSS image of the surrounding field.".format(fig_count, tic)
			
			ptext = '<font size=8>%s</font>' % tess_stars_text
			Story.append(Paragraph(ptext, styles["Normal"]))
		
	
	# --------------------------------------------
	# nearest neighbours
	# --------------------------------------------
	#Story.append(PageBreak()) # always start a new page for this analysis
	if FFI == False:
		im7 = Image(nearest_neighbour_name)
		
		im7._restrictSize(width*0.8, width*0.8)
	
		Story.append(im7)
		fig_count += 1
		Story.append(Spacer(1, 10))
		nn_text = "Fig {}. Lightcurves of the five closest stars to target {} (top pannel). \
			The distances to the target star and the TESS magnitudes are shown for each star. Only ever shown for one sector.".format(fig_count,tic)
		
		ptext = '<font size=8>%s</font>' % nn_text
		Story.append(Paragraph(ptext, styles["Normal"]))
	

	# this plot also only exists if the TPF downloaded sucessfully
	if tpf_corrupt == False:
		# --------------------------------------------
		# pixel_LCs_name
		# --------------------------------------------
		Story.append(Spacer(1, 10))
		im8 = Image(pixel_LCs_name)
		
	
		im8._restrictSize(width*0.65, width*0.65)
	
		Story.append(im8)
		fig_count += 1
		pixLC_text = "Fig {}. Normalised flux extracted for each pixel, using the SPOC pipeline mask, around the time of the transit-like event. \
		The orange/red data points show the in-transit data. The solid red lines show the SPOC pipeline mask. Only shown for one sector.".format(fig_count)
		
		ptext = '<font size=8>%s</font>' % pixLC_text
		Story.append(Paragraph(ptext, styles["Normal"]))
	
	# ------ Phase Folded LC ------


	if len(transit_list) > 1:

		# --------------------------------------------
		# Phase Folded
		# --------------------------------------------
		impf = Image(phasefold_name)

		impf._restrictSize(width*0.35, width*0.35)

		Story.append(impf)
		
		fig_count += 1
		Story.append(Spacer(1, 10))
		flux_aperture_text = "Fig {}. Phase folded lightcurve where the odd and the even transits are shown in different colours. Ensure that the odd and even transits have comparabel shapes and depths.".format(fig_count)
		
		ptext = '<font size=8>%s</font>' % flux_aperture_text
		Story.append(Paragraph(ptext, styles["Normal"]))
		

	# ------ BLS -------
	Story.append(PageBreak()) # always start a new page for this analysis
	# ------

	if bls == True:
		
		Story.append(Spacer(1, 12))
		blsim1 = Image(bls1)
		blsim2 = Image(bls2)

		blsim1._restrictSize(width*0.6, width*0.6)
		blsim2._restrictSize(width*0.6, width*0.6)

		bls_table = (Table([[blsim1, blsim2]],
						colWidths=[width * 0.45], rowHeights=[width * 0.6]))
	
		bls_table.setStyle(TableStyle([('ALIGN', (0, 0), (-1, -1), 'CENTRE'),('VALIGN', (0, 0), (-1,-1), 'MIDDLE')]))
		
		Story.append(bls_table)

		fig_count += 1

		if FFI == False:

			bls1_text = "Fig {}. Box Least Square fitting (BLS) for whole lightcurve binned to 10 minutes. Top left panel: log liklihood periodogram. \
							The solid red line indicates the peak period and the dashed orange lines show the integer \
							harmonics of this period. Middle left panel: Full light curve, unbinned (orange) and binned to 10 minutes (black). \
							The peak period is highlighted by the solid red lines. Bottom left Panel: Phase folded light curve where the found transit-event is fit \
							with a simple box (red line). The pannels on the right show the same diagnostics, however the diagnostic \
							was run with the highest detected signal-to-noise transits, from the initial BLS search, removed. ".format(fig_count)
		
		else:
			bls1_text = "Fig {}. Box Least Square fitting (BLS) for whole lightcurve. Top left panel: log liklihood periodogram. \
							The solid blue line indicates the peak period and the dashed red lines show the integer \
							harmonics of this period. Middle left panel: Full light curve, unbinned LC (orange) . \
							The peak period is highlighted by the solid blue lines. Bottom left Panel: Phase folded light curve where the found transit-event is fit \
							with a simple box (blue line). The pannels on the right show the same diagnostics, however the diagnostic \
							was run with the highest detected signal-to-noise transits, from the initial BLS search, removed. ".format(fig_count)
		

		ptext = '<font size=8>%s</font>' % bls1_text
		Story.append(Paragraph(ptext, styles["Normal"]))
		


		# --------------------
		# ---- BLS TABLE -----

		data_bls= [['Parameter',				  "bls1",														"bls2"],
					   ['period',			    "{:.3f}".format(bls_stats1[0]),	         						 "{:.3f}".format(bls_stats2[0])],
					   ['t0',			        "{:.2f}".format(bls_stats1[1]),	         						 "{:.2f}".format(bls_stats2[1])],
					   ['depth',			    "{:.5f} ± {:.5f}".format(bls_stats1[2][0],bls_stats1[2][1]),	 "{:.5f} ± {:.5f}".format(bls_stats2[2][0],bls_stats2[2][1]) ],
					   ['depth phased',			"{:.5f} ± {:.5f}".format(bls_stats1[3][0],bls_stats1[3][1]),	 "{:.5f} ± {:.5f}".format(bls_stats2[3][0],bls_stats2[3][1]) ],
					   ['depth half',			"{:.5f} ± {:.5f}".format(bls_stats1[4][0],bls_stats1[4][1]),	 "{:.5f} ± {:.5f}".format(bls_stats2[4][0],bls_stats2[4][1]) ],
					   ['depth odd',			"{:.5f} ± {:.5f}".format(bls_stats1[5][0],bls_stats1[5][1]),	 "{:.5f} ± {:.5f}".format(bls_stats2[5][0],bls_stats2[5][1]) ],
					   ['depth even',			"{:.5f} ± {:.5f}".format(bls_stats1[6][0],bls_stats1[6][1]),	 "{:.5f} ± {:.5f}".format(bls_stats2[6][0],bls_stats2[6][1]) ],
					   ]
		
		table_bls=Table(data_bls)
		table_bls=Table(data_bls,colWidths=width * 0.2, style=[
							('LINEABOVE',(0,1),(-1,1),1,colors.black),
							('LINEABOVE',(0,8),(-1,8),1,colors.black),
							('FONTSIZE', (0,0),(-1,7), 8),
							])
	

		# ------ ADD A LINE TO SEPERATE SECTIONS -------
		
		Story.append(Spacer(1, 20))
		line = MCLine(width*0.77)
		Story.append(line)
		
		# ------

		Story.append(Spacer(1, 16))
		ptext = '<font size=10>BLS parameters</font>'
		Story.append(Paragraph(ptext, styles["Normal"]))
		Story.append(Spacer(1, 15))
	
		data_len = len(data_bls)
		
		for each in range(data_len):
			if each % 2 == 0:
				bg_color = colors.whitesmoke
			else:
				bg_color = colors.white
		
			table_bls.setStyle(TableStyle([('BACKGROUND', (0, each), (-1, each), bg_color)]))
		
		
		Story.append(table_bls)
		Story.append(Spacer(1, 15))

		table_count += 1
		Stellartable_text = "Table {}. Summary of the two BLS fits. Fit one is run with the whole lightcurve and fit two is run with the highest detected signal-to-noise transits removed.".format(table_count)
		ptext = '<font size=8>%s</font>' % Stellartable_text
		Story.append(Paragraph(ptext, styles["Normal"]))

		Story.append(PageBreak())
		# -----------


	if model == True:

		#Story.append(PageBreak()) # always start a new page for this analysis
		pyaneti_url = 'https://academic.oup.com/mnras/article/482/1/1017/5094600'

		pyaneti_link = '<link href="%s" color="blue">Pyaneti</link>' % pyaneti_url


		model_title = "Modeling"
		model_text = "The modeling of target TIC {} using the open source {} package.".format(tic, pyaneti_link)
		
		ptext = '<font size=11><b>%s</b></font>' % model_title
		Story.append(Paragraph(ptext, styles["centre"]))
		Story.append(Spacer(1, 10))
		ptext = '<font size=11>%s</font>' % model_text
		Story.append(Paragraph(ptext, styles["centre"]))
		Story.append(Spacer(1, 15))
		
		line = MCLine(width*0.77)
		Story.append(line)

		# --------------------------------------------
		# Pyaneti Modeling results
		# --------------------------------------------
		
		Story.append(Spacer(1, 12))
		im_model = Image(model_name)
			
		im_model._restrictSize(width*0.5, width*0.5)
			
		Story.append(im_model)
		fig_count += 1

		model_text = "Fig {}. The phase folded lightcurve from the the Pyaneti modeling. The solid black line shows the best fit model. See Table 2 for model parameters.".format(fig_count)
			
		ptext = '<font size=8>%s</font>' % model_text
		Story.append(Paragraph(ptext, styles["Normal"]))
		

		# ---------------
		# pyaneti modeling table
	
		# ----------------------------------------------
		# print a table of the model/pyaneti parameters
		# ----------------------------------------------
		
		# this can only be done if the pyaneti model has been run
		# as this takes quite a while this might not always be the case.


		if os.path.exists("{}/{}/model_out/{}_parameters.csv".format(indir, tic, tic)):
			
			Story.append(Spacer(1, 12))
			line = MCLine(width*0.77)
			Story.append(line)
			Story.append(Spacer(1, 12))

			ptext = '<font size=10>Candidate Model Parameters</font>'
			Story.append(Paragraph(ptext, styles["Normal"]))
			Story.append(Spacer(1, 8))
		
			manifest_table = pd.read_csv('{}/{}/model_out/{}_parameters.csv'.format(indir, tic, tic))
		
			#manifest_table
			params = ['T0', 'P', 'e', 'w', 'b', 'a/R*', 'rp/R*', 'Rp', 'Tperi', 'i', 'a', 'Insolation', 'rho*', 'g_p', 'Teq', 'T_tot', 'T_full']
			elements = []
		
			param_vals = []
			param_errs = []
			param_units= []
		
		
			for st in params:
				try:
					s = (manifest_table[manifest_table['Var']==st])
					param_vals.append("{:.3f}".format(s['Val'].values[0] ))
					param_errs.append("+{:.4f}  -{:.4f}".format(s['Pos'].values[0], s['Neg'].values[0]))
					param_units.append("{}".format(s['Unit'].values[0] ))
				except:
					param_vals.append(-999)
					param_errs.append(-999)
					param_units.append(-999)
		
			data_params = [['Parameters', 'Value', 'Uncertainty', 'Unit']]
			for p,v,e,u in zip(params, param_vals, param_errs, param_units):
				data_params.append([p,v,e,u])
		
			table_model=Table(data_params)
			table_model=Table(data_params,colWidths=width * 0.2, style=[
								('LINEABOVE',(0,1),(-1,1),1,colors.black),
								('LINEABOVE',(0,8),(-1,8),1,colors.black),
								('LINEABOVE',(0,len(params)+1),(-1,len(params)+1),1,colors.black),
								('FONTSIZE', (0,0),(-1,len(params)), 8),
								])
		

			data_len = len(data_params)
			
			for each in range(data_len):
				if each % 2 == 0:
					bg_color = colors.whitesmoke
				else:
					bg_color = colors.white
			
				table_model.setStyle(TableStyle([('BACKGROUND', (0, each), (-1, each), bg_color)]))
			

			Story.append(table_model)

			table_count += 1
			Story.append(Spacer(1, 10))
			Stellartable_text = "Table {}. The candidate paramaters from the Pyaneti modeling.".format(table_count)
			ptext = '<font size=8>%s</font>' % Stellartable_text
			Story.append(Paragraph(ptext, styles["Normal"]))
		


	doc.build(Story, onFirstPage=addPageNumber, onLaterPages=addPageNumber)


# ----- ADDITIONAL MODULES ------

class MCLine(Flowable):
	"""
	Line flowable --- draws a line in a flowable
	http://two.pairlist.net/pipermail/reportlab-users/2005-February/003695.html
	"""
 
	#----------------------------------------------------------------------
	def __init__(self, width, height=0):
		Flowable.__init__(self)
		self.width = width
		self.height = height
 
	#----------------------------------------------------------------------
	def __repr__(self):
		return "Line(w=%s)" % self.width
 
	#----------------------------------------------------------------------
	def draw(self):
		"""
		draw the line
		"""
		self.canv.setStrokeColor(grey)
		self.canv.line(0, self.height, self.width, self.height)
	  
class MCLine_color(Flowable):
	"""
	Line flowable --- draws a line in a flowable in COLOUR
	http://two.pairlist.net/pipermail/reportlab-users/2005-February/003695.html
	"""
 
	#----------------------------------------------------------------------
	def __init__(self, width, height=0):
		Flowable.__init__(self)
		self.width = width
		self.height = height
 
	#----------------------------------------------------------------------
	def __repr__(self):
		return "Line(w=%s)" % self.width
 
	#----------------------------------------------------------------------
	def draw(self):
		"""
		draw the line
		"""
		self.canv.setStrokeColor(darkcyan)
		self.canv.line(0, self.height, self.width, self.height)




import numpy as np
from spock import FeatureClassifier
import rebound
from sherlockpipe.nbodies.PlanetInput import PlanetInput


class StabilityCalculator:
    def __init__(self, star_mass):
        self.star_mass = star_mass

    def mass_from_radius(self, radius):
        return radius ** (1 / 0.55) if radius <= 12.1 else radius ** (1 / 0.01)

    def run(self, planet_params):
        sim = rebound.Simulation()
        sim.integrator = "whfast"
        sim.ri_whfast.safe_mode = 0
        sim.dt = 1e-2
        sim.add(m=1.0)
        for planet_param in planet_params:
            sim.add(m=self.mass_from_radius(planet_param.r) * 0.000003003 / self.star_mass, P=planet_param.P,
                    e=planet_param.e, omega=planet_param.omega)
        #sim.status()
        sim.move_to_com()
        model = FeatureClassifier()
        print("SPOCK=" + str(model.predict_stable(sim)))

grid = 5
sc = StabilityCalculator(0.53)
par_e = np.linspace(0.0, 0.7, grid)
par_e1 = np.linspace(0.0, 0.7, grid)
for i in par_e:
    for j in par_e1:
        sc.run([PlanetInput(1.17, 0.01749, 11.76943, i), PlanetInput(1.37, 0.03088, 2.97, j), PlanetInput(2.45, 0, 3.9, 0)])




'''
norm.py: NaN-robust wrappers for mean, median, min, max... + MAD
scatter estimates + simple normalisation routines
'''

import numpy as np

def medsig(array):
    '''
    Return median and outlier-robust estimate of standard deviation
    (1.48 x median of absolute deviations).
    '''
    l = np.isfinite(array)
    if sum(l) == 0:
        return np.nan, np.nan
    if sum(l) == 1:
        return array[l], np.nan
    med = np.median(array[l])
    sig = 1.48 * np.median(abs(array[l] - med))
    return med, sig
  
def mysum(array, axis = None):
    '''
    Return sum of array along specified axis (ignoring NaNs)
    '''
    arr = np.copy(array)
    l = np.isfinite(arr)
    if l.any() == False: return np.nan
    arr[l==False] = 0.0
    res = np.sum(arr, axis = axis)
    if axis != None:
        l_sum = np.sum(l, axis = axis)
        res[l_sum==0] = np.nan
    return res

def mymean(array):
    '''
    Return mean of array (ignoring NaNs)
    '''
    l = np.isfinite(array)
    if l.any() == False: return np.nan
    return(array[l].mean())

def mystd(array):
    '''
    Return standard deviation of array (ignoring NaNs)
    '''
    l = np.isfinite(array)
    if l.any() == False: return np.nan
    return(array[l].std())

def mymin(array):
    '''
    Return minimum of array (ignoring NaNs)
    '''
    l = np.isfinite(array)
    if l.any() == False: return np.nan
    return(array[l].min())

def mymax(array):
    '''
    Return maximum of array (ignoring NaNs)
    '''
    l = np.isfinite(array)
    if l.any() == False: return np.nan
    return(array[l].max())

def myargmin(array):
    '''
    Return index of minimum of array (ignoring NaNs)
    '''
    l = np.isfinite(array)
    if l.any() == False: return -1
    return(array[l].argmin())

def myargmax(array):
    '''
    Return index maximum of array (ignoring NaNs)
    '''
    l = np.isfinite(array)
    if l.any() == False: return -1
    return(array[l].argmax())

def myabs(array):
    l = np.isfinite(array)
    a = np.copy(array)
    a[l] = abs(array[l])
    return a

def norm01(array):
    '''
    Normalize array to zero mean and unit variance
    '''
    med, sig = medsig(array)
    a = np.copy(array)
    a -= med
    a /= sig
    return a, med, sig

def unnorm01(array, med, sig):
    '''
    Unnormalize array
    '''
    a = np.copy(array) 
    a *= sig
    a += med
    return a

def scale01(array):
    '''
    Scale array to range from 0 to 1
    '''
    nmeas, nobj = np.shape(array)
    arr = np.zeros((nmeas, nobj)) + np.nan
    mins = np.zeros(nobj)
    maxs = np.zeros(nobj)
    for i in np.arange(nobj):
        cur = array[:,i]
        mins[i] = mymin(cur)
        maxs[i] = mymax(cur)
        if np.isfinite(mins[i]) * np.isfinite(maxs[i]) * (maxs[i] > mins[i]):
            arr[:,i] = (cur - mins[i]) / (maxs[i] - mins[i])
    return arr, mins, maxs

def unscale(array):
    '''
    Unscale array from 0 to 1 back to original range
    '''
    nmeas, nobj = np.shape(array)
    arr = np.zeros((nmeas, nobj)) + np.nan
    for i in np.arange(nobj):
        arr[:,i] = (array[:,i] * (maxs[i] - mins[i])) + mins[i]
    return arr




from sherlockpipe.objectinfo.ObjectInfo import ObjectInfo


class MissionObjectInfo(ObjectInfo):
    """
    Implementation of ObjectInfo to be used to characterize short-cadence objects from TESS, Kepler and K2 missions.
    """
    def __init__(self, mission_id: str, sectors, initial_mask=None, initial_detrend_period=None):
        """
        @param mission_id: the mission identifier. TIC ##### for TESS, KIC ##### for Kepler and EPIC ##### for K2.
        @param sectors: an array of integers specifying which sectors will be analysed for the object
        @param initial_mask: an array of time ranges provided to mask them into the initial object light curve.
        @param initial_detrend_period: integer value specifying a fixed value for an initial period to be detrended
        from the initial light curve before processing.
        """
        super().__init__(initial_mask, initial_detrend_period)
        self.id = mission_id
        self.sectors = sectors

    def sherlock_id(self):
        return self.id.replace(" ", "") + "_" + str(self.sectors)

    def mission_id(self):
        return self.id



import re
from abc import ABC, abstractmethod


class ObjectInfo(ABC):
    """
    Root class to be extended to characterize the input object to be analysed by Sherlock.
    """
    OBJECT_ID_REGEX = "^(KIC|TIC|EPIC)[-_ ]([0-9]+)$"
    NUMBERS_REGEX = "[0-9]+$"
    MISSION_ID_KEPLER = "KIC"
    MISSION_ID_KEPLER_2 = "EPIC"
    MISSION_ID_TESS = "TIC"
    initial_detrend_period = None
    initial_mask = None

    def __init__(self, initial_mask=None, initial_detrend_period=None):
        self.initial_mask = initial_mask
        self.initial_detrend_period = initial_detrend_period

    @abstractmethod
    def sherlock_id(self):
        """
        Returns the unique name generated for Sherlock processing
        """
        pass

    @abstractmethod
    def mission_id(self):
        """
        Returns the real mission identifier
        """
        pass



from sherlockpipe.objectinfo.ObjectInfo import ObjectInfo

class MissionFfiCoordsObjectInfo(ObjectInfo):
    """
    Implementation of ObjectInfo to be used to characterize long-cadence objects from TESS by providing the RA and Dec.
    """
    def __init__(self, ra, dec, sectors, initial_mask=None, initial_detrend_period=None):
        """
        @param ra: the objects right ascension.
        @param dec: the objects declination.
        @param sectors: an array of integers specifying which sectors will be analysed for the object
        @param initial_mask: an array of time ranges provided to mask them into the initial object light curve.
        @param initial_detrend_period: integer value specifying a fixed value for an initial period to be detrended
        from the initial light curve before processing.
        """
        super().__init__(initial_mask, initial_detrend_period)
        self.ra = ra
        self.dec = dec
        self.sectors = sectors

    def sherlock_id(self):
        return str(self.ra) + "_" + str(self.dec) + "_FFI_" + str(self.sectors)

    def mission_id(self):
        return None





import logging
import numpy as np
from sherlockpipe.star import starinfo
from sherlockpipe.objectinfo.ObjectProcessingError import ObjectProcessingError
from sherlockpipe.objectinfo.preparer.LightcurveBuilder import LightcurveBuilder
import lightkurve as lk


class MissionLightcurveBuilder(LightcurveBuilder):
    def __init__(self):
        super().__init__()

    def build(self, object_info):
        mission_id = object_info.mission_id()
        sherlock_id = object_info.sherlock_id()
        quarters = None
        sectors = None
        logging.info("Retrieving star catalog info...")
        mission, mission_prefix, id = super().parse_object_id(mission_id)
        if mission_prefix not in self.star_catalogs:
            raise ValueError("Wrong object id " + mission_id)
        star_info = starinfo.StarInfo(sherlock_id, *self.star_catalogs[mission_prefix].catalog_info(id))
        logging.info("Downloading lightcurve files...")
        if mission == "TESS" and object_info.sectors != 'all':
            lcf = lk.search_lightcurvefile(str(mission_id), mission=mission, cadence="short", sector=object_info.sectors)\
                .download_all()
        elif mission == "TESS":
            lcf = lk.search_lightcurvefile(str(mission_id), mission=mission, cadence="short").download_all()
        elif object_info.sectors != 'all':
            lcf = lk.search_lightcurvefile(str(mission_id), mission=mission, cadence="short", quarter=object_info.sectors)\
                .download_all()
        else:
            lcf = lk.search_lightcurvefile(str(mission_id), mission=mission, cadence="short").download_all()
        if lcf is None:
            raise ObjectProcessingError("Light curve not found for object id " + mission_id)
        lc = None
        matching_objects = []
        for i in range(0, len(lcf.PDCSAP_FLUX)):
            if lcf.PDCSAP_FLUX[i].label == mission_id:
                if lc is None:
                    lc = lcf.PDCSAP_FLUX[i].normalize()
                else:
                    lc = lc.append(lcf.PDCSAP_FLUX[i].normalize())
            else:
                matching_objects.append(lcf.PDCSAP_FLUX[i].label)
        if len(matching_objects) > 0:
            logging.warning("================================================")
            logging.warning("TICS IN THE SAME PIXEL: " + str(matching_objects))
            logging.warning("================================================")
        lc = lc.remove_nans()
        transits_min_count = 1 if len(lcf) == 0 else 2
        if mission_prefix == self.MISSION_ID_KEPLER or mission_id == self.MISSION_ID_KEPLER_2:
            quarters = [lcfile.quarter for lcfile in lcf]
        elif mission_prefix == self.MISSION_ID_TESS:
            sectors = [file.sector for file in lcf]
        if mission_prefix == self.MISSION_ID_KEPLER_2:
            logging.info("Correcting K2 motion in light curve...")
            quarters = [lcfile.campaign for lcfile in lcf]
            lc = lc.to_corrector("sff").correct(windows=20)
        return lc, star_info, transits_min_count, np.unique(sectors), np.unique(quarters)


class TransitResult:
    def __init__(self, results, period, per_err, duration,
                 t0, depths, depth, count, snr, sde, fap, border_score, in_transit, harmonic=None):
        self.results = results
        self.period = period
        self.per_err = per_err
        self.duration = duration
        self.t0 = t0
        self.depths = depths
        self.depth = depth
        self.count = count
        self.snr = snr
        self.sde = sde
        self.fap = fap
        self.border_score = border_score
        self.in_transit = in_transit
        self.harmonic = harmonic



# from __future__ import print_function, absolute_import, division
import multiprocessing
import re
import shutil
import types
from pathlib import Path

import allesfitter
import numpy as np
import yaml
from argparse import ArgumentParser
from sherlockpipe.star.HabitabilityCalculator import HabitabilityCalculator
import pandas as pd
import os
from os import path
import matplotlib.pyplot as plt


resources_dir = path.join(path.dirname(__file__))


class Fitter:
    def __init__(self, object_dir, only_initial, mcmc = False, detrend = False):
        self.args = types.SimpleNamespace()
        self.args.noshow = True
        self.args.north = False
        self.args.o = True
        self.args.auto = True
        self.args.save = True
        self.args.nickname = ""  # TODO do we set the sherlock id?
        self.args.FFI = False  # TODO get this from input
        self.args.targetlist = "best_signal_latte_input.csv"
        self.args.new_path = ""  # TODO check what to do with this
        self.object_dir = os.getcwd() if object_dir is None else object_dir
        self.latte_dir = str(Path.home()) + "/.sherlockpipe/latte/"
        if not os.path.exists(self.latte_dir):
            os.mkdir(self.latte_dir)
        self.data_dir = self.object_dir
        self.only_initial = only_initial
        self.mcmc = mcmc
        self.detrend = detrend

    def fit(self, candidate_df, star_df, cpus, allesfit_dir):
        candidate_row = candidate_df.iloc[0]
        sherlock_star_file = self.object_dir + "/params_star.csv"
        star_file = allesfit_dir + "/params_star.csv"
        params_file = allesfit_dir + "/params.csv"
        settings_file = allesfit_dir + "/settings.csv"
        if candidate_row["number"] is None or np.isnan(candidate_row["number"]):
            lc_file = "/" + candidate_row["lc"]
        else:
            lc_file = "/" + str(candidate_row["number"]) + "/lc_" + str(candidate_row["curve"]) + ".csv"
        shutil.copyfile(self.object_dir + lc_file, allesfit_dir + "/lc.csv")
        if os.path.exists(sherlock_star_file) and os.path.isfile(sherlock_star_file):
            shutil.copyfile(sherlock_star_file, star_file)
        shutil.copyfile(resources_dir + "/resources/allesfitter/params.csv", params_file)
        shutil.copyfile(resources_dir + "/resources/allesfitter/settings.csv", settings_file)
        fit_width = 0.3333333
        if candidate_row["duration"] is not None:
            fit_width = float(candidate_row["duration"]) / 60 / 24 * 7
        # TODO replace sherlock properties from allesfitter files
        # TODO only use params_star when the star mass or radius was not assumed
        with open(settings_file, 'r+') as f:
            text = f.read()
            text = re.sub('\\${sherlock:cores}', str(cpus), text)
            text = re.sub('\\${sherlock:fit_width}', str(fit_width), text)
            text = re.sub('\\${sherlock:name}', str(candidate_row["name"]), text)
            detrend_param = "baseline_flux_lc,hybrid_offset"
            detrend_param = detrend_param if self.detrend else "#" + detrend_param
            text = re.sub('\\${sherlock:detrend}', detrend_param, text)
            f.seek(0)
            f.write(text)
            f.truncate()
        with open(params_file, 'r+') as f:
            text = f.read()
            text = re.sub('\\${sherlock:t0}', str(candidate_row["t0"]), text)
            text = re.sub('\\${sherlock:period}', str(candidate_row["period"]), text)
            rp_rs = candidate_row["rp_rs"] if candidate_row["rp_rs"] != "-" else 0.1
            text = re.sub('\\${sherlock:rp_rs}', str(rp_rs), text)
            sum_rp_rs_a = (candidate_row["rp_rs"] + star_df.iloc[0]['R_star']) / candidate_row["a"] * 0.00465047 \
                if candidate_row["rp_rs"] != "-" else 0.2
            text = re.sub('\\${sherlock:sum_rp_rs_a}', str(sum_rp_rs_a), text)
            text = re.sub('\\${sherlock:name}', str(candidate_row["name"]), text)
            if os.path.exists(sherlock_star_file) and os.path.isfile(sherlock_star_file):
                text = re.sub('\\${sherlock:ld_a}', str(star_df.iloc[0]["ld_a"]) + ",0", text)
                text = re.sub('\\${sherlock:ld_b}', str(star_df.iloc[0]["ld_b"]) + ",0", text)
            else:
                text = re.sub('\\${sherlock:ld_a}', "0.5,1", text)
                text = re.sub('\\${sherlock:ld_b}', "0.5,1", text)
            f.seek(0)
            f.write(text)
            f.truncate()
        allesfitter.show_initial_guess(allesfit_dir)
        self.custom_plot(candidate_row["name"], candidate_row["period"], fit_width, allesfit_dir, "initial_guess")
        if not self.only_initial:
            if not self.mcmc:
                allesfitter.ns_fit(allesfit_dir)
                allesfitter.ns_output(allesfit_dir)
            elif self.mcmc:
                allesfitter.mcmc_fit(allesfit_dir)
                allesfitter.mcmc_output(allesfit_dir)
            self.custom_plot(candidate_row["name"], candidate_row["period"], fit_width, allesfit_dir, "posterior")

    def custom_plot(self, name, period, fit_width, allesfit_dir, mode="posterior"):
        allesclass = allesfitter.allesclass(allesfit_dir)
        baseline_width = fit_width * 24
        baseline_to_period = fit_width / period
        fig, axes = plt.subplots(2, 3, figsize=(18, 6), gridspec_kw={'height_ratios': [3, 1]}, sharex='col')
        fig.subplots_adjust(left=0.5, right=1.5, hspace=0)
        style = 'full'
        allesclass.plot('lc', name, style, ax=axes[0][0], mode=mode)
        axes[0][0].set_title('lc, ' + style)
        allesclass.plot('lc', name, style + '_residuals', ax=axes[1][0], mode=mode)
        axes[1][0].set_title('')
        style = 'phase'
        allesclass.plot('lc', name, style, ax=axes[0][1], mode=mode, zoomwindow=baseline_to_period)
        axes[0][1].set_title('lc, ' + style)
        axes[0][1].set_xlim([- baseline_to_period / 2, baseline_width / 2])
        allesclass.plot('lc', name, style + '_residuals', ax=axes[1][1], mode=mode, zoomwindow=baseline_to_period)
        axes[1][1].set_title('')
        axes[1][1].set_xlim([- baseline_to_period / 2, baseline_to_period / 2])
        style = 'phasezoom'
        allesclass.plot('lc', name, style, ax=axes[0][2], mode=mode, zoomwindow=baseline_width)
        axes[0][2].set_title('lc, ' + style)
        axes[0][2].set_xlim([- baseline_width / 2, baseline_width / 2])
        allesclass.plot('lc', name, style + '_residuals', ax=axes[1][2], mode=mode, zoomwindow=baseline_width)
        axes[1][2].set_title('')
        axes[1][2].set_xlim([- baseline_width / 2, baseline_width / 2])
        fig.savefig(allesfit_dir + '/results/ns_' + mode + '_' + name + '_custom.pdf', bbox_inches='tight')
        style = ['phasezoom_occ']
        style = ['phase_variations']


if __name__ == '__main__':
    ap = ArgumentParser(description='Fitting of Sherlock objects of interest')
    ap.add_argument('--object_dir',
                    help="If the object directory is not your current one you need to provide the ABSOLUTE path",
                    required=False)
    ap.add_argument('--candidate', type=int, default=None, help="The candidate signal to be used.", required=False)
    ap.add_argument('--only_initial', dest='only_initial', action='store_true',
                        help="Whether to only run an initial guess of the transit")
    ap.set_defaults(only_initial=False)
    ap.add_argument('--cpus', type=int, default=None, help="The number of CPU cores to be used.", required=False)
    ap.add_argument('--mcmc', dest='mcmc', action='store_true', help="Whether to run using mcmc or ns. Default is ns.")
    ap.add_argument('--detrend', dest='detrend', action='store_true', help="Whether to execute detrending in the "
                                                                            "allesfitter runs.")
    ap.add_argument('--properties', help="The YAML file to be used as input.", required=False)
    args = ap.parse_args()
    fitter = Fitter(args.object_dir, args.only_initial, args.mcmc, args.detrend)
    index = 0
    fitting_dir = fitter.data_dir + "/fit_" + str(index)
    while os.path.exists(fitting_dir) or os.path.isdir(fitting_dir):
        fitting_dir = fitter.data_dir + "/fit_" + str(index)
        index = index + 1
    os.mkdir(fitting_dir)
    fitter.data_dir = fitter.object_dir
    star_df = pd.read_csv(fitter.data_dir + "/params_star.csv")
    if args.candidate is None:
        user_properties = yaml.load(open(args.properties), yaml.SafeLoader)
        candidate = pd.DataFrame(columns=['id', 'period', 't0', 'cpus', 'rp_rs', 'a', 'number', 'name', 'lc'])
        candidate = candidate.append(user_properties["planet"], ignore_index=True)
        user_star_df = pd.DataFrame(columns=['R_star', 'M_star'])
        if "star" in user_properties and user_properties["star"] is not None:
            user_star_df = user_star_df.append(user_properties["star"], ignore_index=True)
            if user_star_df.iloc[0]["R_star"] is not None:
                star_df.at[0, "R_star"] = user_star_df.iloc[0]["R_star"]
            if user_star_df.iloc[0]["M_star"] is not None:
                star_df.at[0, "M_star"] = user_star_df.iloc[0]["M_star"]
            if user_star_df.iloc[0]["ld_a"] is not None:
                star_df.at[0, "ld_a"] = user_star_df.iloc[0]["ld_a"]
            if user_star_df.iloc[0]["ld_b"] is not None:
                star_df.at[0, "ld_b"] = user_star_df.iloc[0]["ld_b"]
            if ("a" not in user_properties["planet"] or user_properties["planet"]["a"] is None)\
                    and star_df.iloc[0]["M_star"] is not None and not np.isnan(star_df.iloc[0]["M_star"]):
                candidate.at[0, "a"] = HabitabilityCalculator() \
                    .calculate_semi_major_axis(user_properties["planet"]["period"],
                                               user_properties["star"]["M_star"])
            elif ("a" not in user_properties["planet"] or user_properties["planet"]["a"] is None)\
                    and (star_df.iloc[0]["M_star"] is None or np.isnan(star_df.iloc[0]["M_star"])):
                raise ValueError("Cannot guess semi-major axis without star mass.")
        if candidate.iloc[0]["a"] is None or np.isnan(candidate.iloc[0]["a"]):
            raise ValueError("Semi-major axis is neither provided nor inferred.")
        if candidate.iloc[0]["name"] is None:
            raise ValueError("You need to provide a name for your candidate.")
        if candidate.iloc[0]["lc"] is None:
            raise ValueError("You need to provide a light curve relative path for your candidate.")
        cpus = user_properties["settings"]["cpus"]
    else:
        candidate_selection = int(args.candidate)
        candidates = pd.read_csv(fitter.object_dir + "/candidates.csv")
        if candidate_selection < 1 or candidate_selection > len(candidates.index):
            raise SystemExit("User selected a wrong candidate number.")
        candidates = candidates.rename(columns={'Object Id': 'TICID'})
        candidate = candidates.iloc[[candidate_selection - 1]]
        candidate['number'] = [candidate_selection]
        candidate['name'] = 'SOI_' + candidate['number'].astype(str)
        if args.cpus is None:
            cpus = multiprocessing.cpu_count() - 1
        else:
            cpus = args.cpus
        print("Selected signal number " + str(candidate_selection))
    fitter.fit(candidate, star_df, cpus, fitting_dir)



import wotan
from astropy.stats import sigma_clip
from scipy import stats


class Flattener:
    def __init__(self) -> None:
        super().__init__()

    def flatten_bw(self, flatten_input):
        flatten_lc, trend = wotan.flatten(flatten_input.time, flatten_input.flux, window_length=flatten_input.wl,
                                    return_trend=True, method="biweight", break_tolerance=0.5)
        flatten_lc = sigma_clip(flatten_lc, sigma_lower=20, sigma_upper=3)
        bin_centers_i, bin_means_i, bin_width_i, bin_edges_i, bin_stds_i = \
            self.__compute_flatten_stats(flatten_input.time, flatten_lc, flatten_input.bin_minutes)
        return flatten_lc, trend, bin_centers_i, bin_means_i, flatten_input.wl

    def flatten_gp(self, flatten_input):
        flatten_lc, trend = wotan.flatten(flatten_input.time, flatten_input.flux, method="gp", kernel='matern',
                                               kernel_size=flatten_input.wl, return_trend=True, break_tolerance=0.5)
        flatten_lc = sigma_clip(flatten_lc, sigma_lower=20, sigma_upper=3)
        bin_centers_i, bin_means_i, bin_width_i, bin_edges_i, bin_stds_i = \
            self.__compute_flatten_stats(flatten_input.time, flatten_lc, flatten_input.bin_minutes)
        return flatten_lc, trend, bin_centers_i, bin_means_i, flatten_input.wl

    def __compute_flatten_stats(self, time, flux, bin_minutes):
        bins_i = len(time) * 2 / bin_minutes
        bin_means_i, bin_edges_i, binnumber_i = stats.binned_statistic(time, flux, statistic='mean', bins=bins_i)
        bin_stds_i, _, _ = stats.binned_statistic(time, flux, statistic='std', bins=bins_i)
        bin_width_i = (bin_edges_i[1] - bin_edges_i[0])
        bin_centers_i = bin_edges_i[1:] - bin_width_i / 2
        return bin_centers_i, bin_means_i, bin_width_i, bin_edges_i, bin_stds_i

class FlattenInput:
    def __init__(self, time, flux, wl, bin_minutes):
        self.time = time
        self.flux = flux
        self.wl = wl
        self.bin_minutes = bin_minutes


# from __future__ import print_function, absolute_import, division
import shutil
import types
from pathlib import Path
import traceback
import lightkurve
import numpy as np
import matplotlib
import matplotlib.pyplot as plt
import matplotlib.gridspec as gridspec
import yaml
from matplotlib.colorbar import Colorbar
from matplotlib import patches
from astropy.visualization.mpl_normalize import ImageNormalize
from astropy.table import Table
from astropy.io import ascii
import astropy.visualization as stretching
from argparse import ArgumentParser
from sherlockpipe import tpfplotter
import six
import sys
import sherlockpipe.LATTE
sys.modules['astropy.extern.six'] = six
sys.modules['LATTE'] = sherlockpipe.LATTE

matplotlib.use('Agg')
import pandas as pd
import os
from os.path import exists
import ast
import csv
from sherlockpipe.LATTE import LATTEutils, LATTEbrew
from os import path

'''WATSON: Verboseless Vetting and Adjustments of Transits for Sherlock Objects of iNterest
This class intends to provide a inspection and transit fitting tool for SHERLOCK Candidates.
'''
# get the system path
syspath = str(os.path.abspath(LATTEutils.__file__))[0:-14]
# ---------

# --- IMPORTANT TO SET THIS ----
out = 'pipeline'  # or TALK or 'pipeline'
ttran = 0.1
resources_dir = path.join(path.dirname(__file__))

class Vetter:
    def __init__(self, object_dir):
        self.args = types.SimpleNamespace()
        self.args.noshow = True
        self.args.north = False
        self.args.o = True
        self.args.mpi = False
        self.args.auto = True
        self.args.save = True
        self.args.nickname = ""  # TODO do we set the sherlock id?
        self.args.FFI = False  # TODO get this from input
        self.args.targetlist = "best_signal_latte_input.csv"
        self.args.new_path = ""  # TODO check what to do with this
        self.object_dir = os.getcwd() if object_dir is None else object_dir
        self.latte_dir = str(Path.home()) + "/.sherlockpipe/latte/"
        if not os.path.exists(self.latte_dir):
            os.mkdir(self.latte_dir)
        self.data_dir = self.object_dir

    def update(self):
        indir = self.latte_dir
        if os.path.exists(indir) and os.path.isdir(indir):
            shutil.rmtree(indir, ignore_errors=True)
        os.makedirs(indir)
        with open("{}/_config.txt".format(indir), 'w') as f:
            f.write(str(indir))
        print("\n Download the text files required ... ")
        print("\n Only the manifest text files (~325 M) will be downloaded and no TESS data.")
        print("\n This step may take a while but luckily it only has to run once... \n")
        if not os.path.exists("{}".format(indir)):
            os.makedirs(indir)
        if not os.path.exists("{}/data".format(indir)):
            os.makedirs("{}/data".format(indir))
        outF = open(indir + "/data/temp.txt", "w")
        outF.write("#all LC file links")
        outF.close()
        outF = open(indir + "/data/temp_tp.txt", "w+")
        outF.write("#all LC file links")
        outF.close()
        LATTEutils.data_files(indir)
        LATTEutils.tp_files(indir)
        LATTEutils.TOI_TCE_files(indir)
        LATTEutils.momentum_dumps_info(indir)

    def __prepare(self, candidate_df):
        """
        Downloads and fills files to be used by LATTE analysis.
        @return: the latte used directory, an open df to be used by LATTE analysis and the tic selected
        """
        # check whether a path already exists
        indir = self.latte_dir
        # SAVE the new output path
        if not os.path.exists("{}/_config.txt".format(indir)):
            self.update()
        candidate_df['TICID'] = candidate_df['TICID'].str.replace("TIC ", "")
        TIC_wanted = list(set(candidate_df['TICID']))
        nlc = len(TIC_wanted)
        print("nlc length: {}".format(nlc))
        print('{}/manifest.csv'.format(str(indir)))
        if exists('{}/manifest.csv'.format(str(indir))):
            print("Existing manifest file found, will skip previously processed LCs and append to end of manifest file")
        else:
            print("Creating new manifest file")
            metadata_header = ['TICID', 'Marked Transits', 'Sectors', 'RA', 'DEC', 'Solar Rad', 'TMag', 'Teff',
                               'thissector', 'TOI', 'TCE', 'TCE link', 'EB', 'Systematics', 'Background Flux',
                               'Centroids Positions', 'Momentum Dumps', 'Aperture Size', 'In/out Flux', 'Keep',
                               'Comment', 'starttime']
            with open('{}/manifest.csv'.format(str(indir)), 'w') as f:  # save in the photometry folder
                writer = csv.writer(f, delimiter=',')
                writer.writerow(metadata_header)
        return indir, candidate_df, TIC_wanted, candidate_df.iloc[0]["ffi"]

    def __process(self, indir, tic, sectors_in, transit_list, t0, period, ffi):
        """
        Performs the LATTE analysis to generate PNGs and also the TPFPlotter analysis to get the field of view
        information.
        @param indir: the vetting source and resources directory
        @param tic: the tic to be processed
        @param sectors_in: the sectors to be used for the given tic
        @param transit_list: the list of transits for the given tic
        @return: the given tic
        """
        sectors_all, ra, dec = LATTEutils.tess_point(indir, tic)
        try:
            sectors = list(set(sectors_in) & set(sectors_all))
            if len(sectors) == 0:
                print("The target was not observed in the sector(s) you stated ({}). \
                        Therefore take all sectors that it was observed in: {}".format(sectors, sectors_all))
                sectors = sectors_all
        except:
            sectors = sectors_all

        if not ffi:
            alltime, allflux, allflux_err, all_md, alltimebinned, allfluxbinned, allx1, allx2, ally1, ally2, alltime12, allfbkg, start_sec, end_sec, in_sec, tessmag, teff, srad = LATTEutils.download_data(
                indir, sectors, tic)
        else:
            alltime_list, allflux, allflux_small, allflux_flat, all_md, allfbkg, allfbkg_t, start_sec, end_sec, in_sec, X1_list, X4_list, apmask_list, arrshape_list, tpf_filt_list, t_list, bkg_list, tpf_list = LATTEutils.download_data_FFI(indir, sectors, syspath, sectors_all, tic, True)
            srad = "-"
            tessmag = "-"
            teff = "-"
            alltime = alltime_list
        simple = False
        BLS = False
        model = False
        save = True
        DV = True
        # TODO decide whether to use transit_list or period
        transit_list = []
        last_time = alltime[len(alltime) - 1]
        num_of_transits = int((last_time - t0) / period)
        transit_lists = t0 + period * range(0, num_of_transits)
        transit_lists = [transit_lists[x:x + 3] for x in range(0, len(transit_lists), 3)]
        for index, transit_list in enumerate(transit_lists):
            transit_results_dir = self.data_dir + "/" + str(index)
            try:
                if not ffi:
                    LATTEbrew.brew_LATTE(tic, indir, syspath, transit_list, simple, BLS, model, save, DV, sectors,
                                         sectors_all,
                                         alltime, allflux, allflux_err, all_md, alltimebinned, allfluxbinned, allx1, allx2,
                                         ally1, ally2, alltime12, allfbkg, start_sec, end_sec, in_sec, tessmag, teff, srad, ra,
                                         dec, self.args)
                else:
                    LATTEbrew.brew_LATTE_FFI(tic, indir, syspath, transit_list, simple, BLS, model, save, DV, sectors,
                                             sectors_all, alltime, allflux_flat, allflux_small, allflux, all_md, allfbkg,
                                             allfbkg_t, start_sec, end_sec, in_sec, X1_list, X4_list, apmask_list,
                                             arrshape_list, tpf_filt_list, t_list, bkg_list, tpf_list, ra, dec, self.args)
                # LATTE_DV.LATTE_DV(tic, indir, syspath, transit_list, sectors_all, simple, BLS, model, save, DV, sectors,
                #                      sectors_all,
                #                      alltime, allflux, allflux_err, all_md, alltimebinned, allfluxbinned, allx1, allx2,
                #                      ally1, ally2, alltime12, allfbkg, start_sec, end_sec, in_sec, tessmag, teff, srad, ra,
                #                      dec, self.args)
                tp_downloaded = True
                shutil.move(vetter.latte_dir + "/" + tic, transit_results_dir)
            except Exception as e:
                traceback.print_exc()
                # see if it made any plots - often it just fails on the TPs as they are very large
                if exists("{}/{}/{}_fullLC_md.png".format(indir, tic, tic)):
                    print("couldn't download TP but continue anyway")
                    tp_downloaded = False
                    shutil.move(vetter.latte_dir + "/" + tic, transit_results_dir)
                else:
                    continue
        # check again whether the TPs downloaded - depending on where the code failed it might still have worked.
        if exists("{}/{}/{}_aperture_size.png".format(indir, tic, tic)):
            tp_downloaded = True
        else:
            tp_downloaded = False
            print("code ran but no TP -- continue anyway")
        # -------------
        # check whether it's a TCE or a TOI

        # TCE -----
        lc_dv = np.genfromtxt('{}/data/tesscurl_sector_all_dv.sh'.format(indir), dtype=str)
        TCE_links = []
        for i in lc_dv:
            if str(tic) in str(i[6]):
                TCE_links.append(i[6])
        if len(TCE_links) == 0:
            TCE = " - "
            TCE = False
        else:
            TCE_links = np.sort(TCE_links)
            TCE_link = TCE_links[0]  # this link should allow you to acess the MAST DV report
            TCE = True
        # TOI -----
        TOI_planets = pd.read_csv('{}/data/TOI_list.txt'.format(indir), comment="#")
        TOIpl = TOI_planets.loc[TOI_planets['TIC'] == float(tic)]
        TOI = False
        # TODO check why TOI is useful
        # else:
        #     TOI = True
        #     TOI_name = (float(TOIpl["Full TOI ID"]))
        # -------------
        # return the tic so that it can be stored in the manifest to keep track of which files have already been produced
        # and to be able to skip the ones that have already been processed if the code has to be restarted.
        mnd = {}
        mnd['TICID'] = tic
        mnd['MarkedTransits'] = transit_list
        mnd['Sectors'] = sectors_all
        mnd['RA'] = ra
        mnd['DEC'] = dec
        mnd['SolarRad'] = srad
        mnd['TMag'] = tessmag
        mnd['Teff'] = teff
        mnd['thissector'] = sectors
        # make empty fields for the test to be checked
        if TOI == True:
            mnd['TOI'] = TOI_name
        else:
            mnd['TOI'] = " "
        if TCE == True:
            mnd['TCE'] = "Yes"
            mnd['TCE_link'] = TCE_link
        else:
            mnd['TCE'] = " "
            mnd['TCE_link'] = " "
        mnd['EB'] = " "
        mnd['Systematics'] = " "
        mnd['TransitShape'] = " "
        mnd['BackgroundFlux'] = " "
        mnd['CentroidsPositions'] = " "
        mnd['MomentumDumps'] = " "
        mnd['ApertureSize'] = " "
        mnd['InoutFlux'] = " "
        mnd['Keep'] = " "
        mnd['Comment'] = " "
        mnd['starttime'] = np.nanmin(alltime) if not isinstance(alltime, str) else "-"
        return mnd

    def vetting(self, candidate):
        indir, df, TIC_wanted, ffi = self.__prepare(candidate)
        for tic in TIC_wanted:
            # check the existing manifest to see if I've processed this file!
            manifest_table = pd.read_csv('{}/manifest.csv'.format(str(indir)))
            # get a list of the current URLs that exist in the manifest
            urls_exist = manifest_table['TICID']
            # get the transit time list
            period = df.loc[df['TICID'] == tic]['period'].iloc[0]
            t0 = df.loc[df['TICID'] == tic]['t0'].iloc[0]
            transit_list = ast.literal_eval(((df.loc[df['TICID'] == tic]['transits']).values)[0])
            try:
                sectors_in = ast.literal_eval(str(((df.loc[df['TICID'] == tic]['sectors']).values)[0]))
                if (type(sectors_in) == int) or (type(sectors_in) == float):
                    sectors = [sectors_in]
                else:
                    sectors = list(sectors_in)
            except:
                sectors = [0]
            index = 0
            vetting_dir = self.data_dir + "/vetting_" + str(index)
            while os.path.exists(vetting_dir) or os.path.isdir(vetting_dir):
                vetting_dir = self.data_dir + "/vetting_" + str(index)
                index = index + 1
            os.mkdir(vetting_dir)
            self.data_dir = vetting_dir
            res = self.__process(indir, tic, sectors, transit_list, t0, period, ffi)
            if res['TICID'] == -99:
                print('something went wrong')
                continue
            self.vetting_field_of_view(indir, tic, res['RA'], res['DEC'], sectors)
            shutil.move(vetter.latte_dir + "/tpfplot", vetting_dir + "/tpfplot")
            # TODO improve this condition to check whether tic, sectors and transits exist
            if not np.isin(tic, urls_exist):
                # make sure the file is opened as append only
                with open('{}/manifest.csv'.format(str(indir)), 'a') as tic:  # save in the photometry folder
                    writer = csv.writer(tic, delimiter=',')
                    metadata_data = [res['TICID']]
                    metadata_data.append(res['MarkedTransits'])
                    metadata_data.append(res['Sectors'])
                    metadata_data.append(res['RA'])
                    metadata_data.append(res['DEC'])
                    metadata_data.append(res['SolarRad'])
                    metadata_data.append(res['TMag'])
                    metadata_data.append(res['Teff'])
                    metadata_data.append(res['thissector'])
                    metadata_data.append(res['TOI'])
                    metadata_data.append(res['TCE'])
                    metadata_data.append(res['TCE_link'])
                    metadata_data.append(res['EB'])
                    metadata_data.append(res['Systematics'])
                    metadata_data.append(res['BackgroundFlux'])
                    metadata_data.append(res['CentroidsPositions'])
                    metadata_data.append(res['MomentumDumps'])
                    metadata_data.append(res['ApertureSize'])
                    metadata_data.append(res['InoutFlux'])
                    metadata_data.append(res['Keep'])
                    metadata_data.append(res['Comment'])
                    metadata_data.append(res['starttime'])
                    writer.writerow(metadata_data)
        return TIC_wanted

    def vetting_field_of_view(self, indir, tic, ra, dec, sectors):
        maglim = 6
        sectors_search = None if sectors is not None and len(sectors) == 0 else sectors
        tpf_source = lightkurve.search_targetpixelfile("TIC " + str(tic), sector=sectors, mission='TESS')
        if tpf_source is None or len(tpf_source) == 0:
            ra_str = str(ra)
            dec_str = "+" + str(dec) if dec >= 0 else str(dec)
            tpf_source = lightkurve.search_tesscut(ra_str + " " + dec_str, sector=sectors_search)
        for i in range(0, len(tpf_source)):
            tpf = tpf_source[i].download(cutout_size=(12, 12))
            pipeline = True
            fig = plt.figure(figsize=(6.93, 5.5))
            gs = gridspec.GridSpec(1, 3, height_ratios=[1], width_ratios=[1, 0.05, 0.01])
            gs.update(left=0.05, right=0.95, bottom=0.12, top=0.95, wspace=0.01, hspace=0.03)
            ax1 = plt.subplot(gs[0, 0])
            # TPF plot
            mean_tpf = np.mean(tpf.flux, axis=0)
            nx, ny = np.shape(mean_tpf)
            norm = ImageNormalize(stretch=stretching.LogStretch())
            division = np.int(np.log10(np.nanmax(tpf.flux)))
            splot = plt.imshow(np.nanmean(tpf.flux, axis=0) / 10 ** division, norm=norm, \
                               extent=[tpf.column, tpf.column + ny, tpf.row, tpf.row + nx], origin='bottom', zorder=0)
            # Pipeline aperture
            if pipeline:  #
                aperture_mask = tpf.pipeline_mask
                aperture = tpf._parse_aperture_mask(aperture_mask)
                maskcolor = 'lightgray'
                print("    --> Using pipeline aperture...")
            else:
                aperture_mask = tpf.create_threshold_mask(threshold=10, reference_pixel='center')
                aperture = tpf._parse_aperture_mask(aperture_mask)
                maskcolor = 'lightgray'
                print("    --> Using threshold aperture...")

            for i in range(aperture.shape[0]):
                for j in range(aperture.shape[1]):
                    if aperture_mask[i, j]:
                        ax1.add_patch(patches.Rectangle((j + tpf.column, i + tpf.row),
                                                        1, 1, color=maskcolor, fill=True, alpha=0.4))
                        ax1.add_patch(patches.Rectangle((j + tpf.column, i + tpf.row),
                                                        1, 1, color=maskcolor, fill=False, alpha=1, lw=2))
            # Gaia sources
            gaia_id, mag = tpfplotter.get_gaia_data_from_tic(tic)
            r, res = tpfplotter.add_gaia_figure_elements(tpf, magnitude_limit=mag + np.float(maglim), targ_mag=mag)
            x, y, gaiamags = r
            x, y, gaiamags = np.array(x) + 0.5, np.array(y) + 0.5, np.array(gaiamags)
            size = 128.0 / 2 ** ((gaiamags - mag))
            plt.scatter(x, y, s=size, c='red', alpha=0.6, edgecolor=None, zorder=10)
            # Gaia source for the target
            this = np.where(np.array(res['Source']) == int(gaia_id))[0]
            plt.scatter(x[this], y[this], marker='x', c='white', s=32, zorder=11)
            # Legend
            add = 0
            if np.int(maglim) % 2 != 0:
                add = 1
            maxmag = np.int(maglim) + add
            legend_mags = np.linspace(-2, maxmag, np.int((maxmag + 2) / 2 + 1))
            fake_sizes = mag + legend_mags  # np.array([mag-2,mag,mag+2,mag+5, mag+8])
            for f in fake_sizes:
                size = 128.0 / 2 ** ((f - mag))
                plt.scatter(0, 0, s=size, c='red', alpha=0.6, edgecolor=None, zorder=10,
                            label=r'$\Delta m=$ ' + str(np.int(f - mag)))
            ax1.legend(fancybox=True, framealpha=0.7)
            # Source labels
            dist = np.sqrt((x - x[this]) ** 2 + (y - y[this]) ** 2)
            dsort = np.argsort(dist)
            for d, elem in enumerate(dsort):
                if dist[elem] < 6:
                    plt.text(x[elem] + 0.1, y[elem] + 0.1, str(d + 1), color='white', zorder=100)
            # Orientation arrows
            tpfplotter.plot_orientation(tpf)
            # Labels and titles
            plt.xlim(tpf.column, tpf.column + ny)
            plt.ylim(tpf.row, tpf.row + nx)
            plt.xlabel('Pixel Column Number', fontsize=16)
            plt.ylabel('Pixel Row Number', fontsize=16)
            plt.title('Coordinates ' + tic + ' - Sector ' + str(tpf.sector),
                      fontsize=16)  # + ' - Camera '+str(tpf.camera))  #
            # Colorbar
            cbax = plt.subplot(gs[0, 1])  # Place it where it should be.
            pos1 = cbax.get_position()  # get the original position
            pos2 = [pos1.x0 - 0.05, pos1.y0, pos1.width, pos1.height]
            cbax.set_position(pos2)  # set a new position
            cb = Colorbar(ax=cbax, mappable=splot, orientation='vertical', ticklocation='right')
            plt.xticks(fontsize=14)
            exponent = r'$\times 10^' + str(division) + '$'
            cb.set_label(r'Flux ' + exponent + r' (e$^-$)', labelpad=10, fontsize=16)
            save_dir = indir + "/tpfplot"
            if not os.path.exists(save_dir):
                os.makedirs(save_dir)
            plt.savefig(save_dir + '/TPF_Gaia_TIC' + tic + '_S' + str(tpf.sector) + '.pdf')
            # Save Gaia sources info
            dist = np.sqrt((x - x[this]) ** 2 + (y - y[this]) ** 2)
            GaiaID = np.array(res['Source'])
            srt = np.argsort(dist)
            x, y, gaiamags, dist, GaiaID = x[srt], y[srt], gaiamags[srt], dist[srt], GaiaID[srt]
            IDs = np.arange(len(x)) + 1
            inside = np.zeros(len(x))
            for i in range(aperture.shape[0]):
                for j in range(aperture.shape[1]):
                    if aperture_mask[i, j]:
                        xtpf, ytpf = j + tpf.column, i + tpf.row
                        _inside = np.where((x > xtpf) & (x < xtpf + 1) &
                                           (y > ytpf) & (y < ytpf + 1))[0]
                        inside[_inside] = 1
            data = Table([IDs, GaiaID, x, y, dist, dist * 21., gaiamags, inside.astype('int')],
                         names=['# ID', 'GaiaID', 'x', 'y', 'Dist_pix', 'Dist_arcsec', 'Gmag', 'InAper'])
            ascii.write(data, save_dir + '/Gaia_TIC' + tic + '_S' + str(tpf.sector) + '.dat', overwrite=True)

    def show_candidates(self):
        self.candidates = pd.read_csv(self.object_dir + "/candidates.csv")
        self.candidates.index = np.arange(1, len(self.candidates) + 1)
        print("Suggested candidates are:")
        print(self.candidates.to_markdown(index=True))
        pass

    def demand_candidate_selection(self):
        user_input = input("Select candidate number to be examined and fit: ")
        if user_input.startswith("q"):
            raise SystemExit("User quitted")
        self.candidate_selection = int(user_input)
        if self.candidate_selection < 1 or self.candidate_selection > len(self.candidates.index):
            raise SystemExit("User selected a wrong candidate number.")
        self.data_dir = self.object_dir + "/" + str(self.candidate_selection)


if __name__ == '__main__':
    ap = ArgumentParser(description='Vetting of Sherlock objects of interest')
    ap.add_argument('--object_dir', help="If the object directory is not your current one you need to provide the ABSOLUTE path", required=False)
    ap.add_argument('--candidate', type=int, default=None, help="The candidate signal to be used.", required=False)
    ap.add_argument('--properties', help="The YAML file to be used as input.", required=False)
    args = ap.parse_args()
    vetter = Vetter(args.object_dir)
    if args.candidate is None:
        user_properties = yaml.load(open(args.properties), yaml.SafeLoader)
        candidate = pd.DataFrame(columns=['id', 'transits', 'sectors', 'FFI'])
        candidate = candidate.append(user_properties, ignore_index=True)
        candidate = candidate.rename(columns={'id': 'TICID'})
        candidate['TICID'] = candidate["TICID"].apply(str)
    else:
        candidate_selection = int(args.candidate)
        candidates = pd.read_csv(vetter.object_dir + "/candidates.csv")
        if candidate_selection < 1 or candidate_selection > len(candidates.index):
            raise SystemExit("User selected a wrong candidate number.")
        candidates = candidates.rename(columns={'Object Id': 'TICID'})
        candidate = candidates.iloc[[candidate_selection - 1]]
        vetter.data_dir = vetter.object_dir
        print("Selected signal number " + str(candidate_selection))
        vetter.vetting(candidate)



import os
import sys
import time 
import datetime
import unittest
from dateutil import parser


import warnings
warnings.filterwarnings("ignore")


import LATTE.LATTEutils as utils

# test the downloading of the data
# get the outir (or indir) path
syspath = str(os.path.abspath(utils.__file__))[0:-14]

indir = "./LATTE/test_output"    

# -----------------------------
'''
Test whether the BLS algorithm works correctly and is able to plot. 

'''

# -------------
# test with these input parameters (this data is already downloaded so it's only testing that the plotting part works)
tic = '55525572'
sector = '5'
sectors = [5]
transit_list = [1454.7]
transit_sec = '5'
# -------------


# create a mock 'argparser' becasuse the argparse function doesn't work within the unittest module 
# for now use all of the default values. 
class Namespace:
    def __init__(self, **kwargs):  # arbitrary number of input values
        self.__dict__.update(kwargs)

# test it with the default arguments (except noshow - we don't want the plots to show up in the test run)
global args
args = Namespace(new_data=False, tic='no',sector='no', targetlist='no', 
    noshow=True, o=False, auto=False,nickname='noname', FFI=False, save=True,north=False,new_path=False,mpi=False)
# --------------

class TestBoxLeastSquareTest(unittest.TestCase):
    '''
    Test the extraction of the information from the TP file (already on the system)
    '''    
    def test_BLS(self):

        # get data needed to run the BLS - use other unittest to make sure that te data handling works as expected to get to this point. 

        alltime, allflux, allflux_err, all_md, alltimebinned, allfluxbinned, allx1, allx2, ally1, ally2, alltime12, allfbkg, start_sec, end_sec, in_sec, tessmag, teff, srad = utils.download_data(indir,sector, tic, binfac = 5, test = './LATTE/tests/tic55525572_lc.fits')

        # run the BLS
        # the plotting is called from within this 
        bls_stats1, bls_stats2 = utils.data_bls(tic, indir, alltime, allflux, allfluxbinned, alltimebinned, args)

        # this function should output a plot tahts hows the detrending, and two BLS plots. 
        # Ensure that these are generated and that the number are what we expect them to be. 

        #these are the paths of the files that should have bee created
        BLS1_path = '{}/55525572/55525572_bls_first.png'.format(indir)
        BLS2_path = '{}/55525572/55525572_bls_second.png'.format(indir)

        # Get file's Last modification time stamp only in terms of seconds since epoch 
        time_created_BLS1= os.path.getmtime(BLS1_path)
        time_created_BLS2= os.path.getmtime(BLS2_path)

        # Convert seconds since epoch to readable timestamp
        t_create_BLS1 = parser.parse(time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time_created_BLS1)))
        t_create_BLS2 = parser.parse(time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time_created_BLS2)))
      
        # the time now (needed to get the time since creation)
        t_now = (datetime.datetime.now())

        # -------
        # time difference in minutes
        time_since_creation_BLS1 =  ((t_now - t_create_BLS1).seconds / 60)
        time_since_creation_BLS2 =  ((t_now - t_create_BLS2).seconds / 60)

        self.assertLess(time_since_creation_BLS1, 1, "No BLS plot generated in the last 60 seconds") # a less than b

        # check that the output numbers make sense
        self.assertAlmostEqual(float(bls_stats1[0]), float(16.910000000000014), places=5)
        self.assertAlmostEqual(float(bls_stats1[1]), float(0.3901880448498858), places=5)

        self.assertAlmostEqual(float(bls_stats2[0]), float(0.51))
        self.assertAlmostEqual(float(bls_stats2[1]), float( 0.305186334480843), places=5)


if __name__ == '__main__':

    unittest.main()






from contextlib import contextmanager
from timeit import default_timer
from sherlockpipe.sherlock import Sherlock
from sherlockpipe.objectinfo.MissionObjectInfo import MissionObjectInfo


@contextmanager
def elapsed_timer():
    start = default_timer()
    elapser = lambda: str(default_timer() - start)
    yield lambda: elapser()
    end = default_timer()
    elapser = lambda: str(end - start)


with elapsed_timer() as elapsed:
    # We will use only one object id so we can explain better the detrend configs that the coder can select
    # We will:
    # 1 Enable the initial smooth function, which reduces local noise in the signal.
    # 2 Enable the initial High RMS areas masking. This procedure will mask all the lightcurve time binned ranges by
    # the 'initial_rms_bin_hours' value with a threshold of the 'initial_rms_threshold' value * RMS_median.
    # 3 Set the number of detrends to be done from PDCSAP_FLUX for each run.
    # 4 Set the SHERLOCK PDCSAP_FLUX detrends method to Gaussian Processes.
    # 5 Set the number of CPU cores to be used by the detrending procedure.
    # 6 Enable the Auto-Detrend detection, which will search for strong periodicities in the light curve and do an
    # initial detrend for it based on the selected 'auto_detrend_method' method and the value of the
    # 'auto_detrend_ratio' value, which ensures that we are detrending the light curve at 'auto_detrend_ratio' times
    # the stronger period.
    sherlock = Sherlock([MissionObjectInfo("TIC 181804752", 'all')])\
        .setup_detrend(initial_smooth=True, initial_rms_mask=True, initial_rms_threshold=2.5, initial_rms_bin_hours=3,
                       n_detrends=12, detrend_method="gp", cores=2, auto_detrend_periodic_signals=True,
                       auto_detrend_ratio=1/3, auto_detrend_method="cosine")\
        .run()
    print("Analysis took " + elapsed() + "s")



import rebound
import numpy as np
import matplotlib.pyplot as plt
from IPython.display import display, clear_output
from sherlockpipe.nbodies.PlanetInput import PlanetInput


class StabilityCalculator:
    def __init__(self, star_mass):
        self.star_mass = star_mass

    def mass_from_radius(self, radius):
        return radius ** (1 / 0.55) if radius <= 12.1 else radius ** (1 / 0.01)

    def run(self, planet_params):
        sim = rebound.Simulation()
        sim.integrator = "whfast"
        sim.ri_whfast.safe_mode = 0
        sim.dt = 1e-2
        sim.add(m=1.0)
        for planet_param in planet_params:
            sim.add(m=self.mass_from_radius(planet_param.r) * 0.000003003 / self.star_mass, a=planet_param.a, e=planet_param.e, omega=planet_param.omega)
        #sim.status()
        sim.move_to_com()
        sim.init_megno()
        sim.exit_max_distance = 20.
        try:
            sim.integrate(5e2 * 2. * np.pi, exact_finish_time=0) # integrate for 500 years, integrating to the nearest
            # for i in range(500):
            #     sim.integrate(sim.t + i * 2 * np.pi)
            #     fig, ax = rebound.OrbitPlot(sim, color=True, unitlabel="[AU]", xlim=[-0.1, 0.1], ylim=[-0.1, 0.1])
            #     plt.show()
            #     plt.close(fig)
                #clear_output(wait=True)
            #timestep for each output to keep the timestep constant and preserve WHFast's symplectic nature
            megno = sim.calculate_megno()
            megno = megno if megno < 10 else 10
            return megno
        except rebound.Escape:
            return 10. # At least one particle got ejected, returning large MEGNO

planet_params = []
parameters = []
grid = 20
par_e = np.linspace(0.0, 0.7, grid)
par_e1 = np.linspace(0.0, 0.7, grid)
for i in par_e:
    for j in par_e1:
        parameters.append((PlanetInput(0.01749, 11.76943, i), PlanetInput(0.03088, 2.97, j)))
from rebound.interruptible_pool import InterruptiblePool

pool = InterruptiblePool()
results = pool.map(StabilityCalculator(0.512881).run, parameters)
results2d = np.array(results).reshape(grid, grid)
fig = plt.figure(figsize=(7, 5))
ax = plt.subplot(111)
extent = [min(par_e), max(par_e), min(par_e1), max(par_e1)]
ax.set_xlim(extent[0], extent[1])
ax.set_xlabel("ecc1 $e$")
ax.set_ylim(extent[2], extent[3])
ax.set_ylabel("ecc2 $e1$")
im = ax.imshow(results2d, interpolation="none", vmin=1.9, vmax=10, cmap="RdYlGn_r", origin="lower", aspect='auto', extent=extent)
cb = plt.colorbar(im, ax=ax)
cb.set_label("MEGNO $\\langle Y \\rangle$")
plt.show()



import os
import shutil
import time
from argparse import ArgumentParser

import eleanor
from sherlockpipe.vet import Vetter
from sherlockpipe.ois.OisManager import OisManager


class Updater:
    """
    This class should be used to download SHERLOCK data as objects of interest, ELEANOR or LATTE metadata. It stores
    timestamps to remind when was the last time it updated and if called again, it could do nothing if it reads very
    recent operations.
    """
    def __init__(self):
        pass

    def update(self, clean, ois, force):
        """
        The main method of the update task. Reloads OIs, ELEANOR and LATTE metadata.
        @param clean: Specifies whether the ELEANOR data should be wiped and downloaded again.
        @param ois: Specifies whether the OIs metadata is the only one to be refreshed (ignoring ELEANOR and LATTE)
        @param force: Specifies whether the last download timestamp should be ignored and proceed as if a refresh was
        needed.
        """
        ois_manager = OisManager()
        timestamp_ois_path = os.path.join(os.path.expanduser('~'), '.sherlockpipe/timestamp_ois.txt')
        timestamp_eleanor_path = os.path.join(os.path.expanduser('~'), '.sherlockpipe/timestamp_eleanor.txt')
        timestamp_latte_path = os.path.join(os.path.expanduser('~'), '.sherlockpipe/timestamp_latte.txt')
        ois_timestamp = 0
        eleanor_timestamp = 0
        latte_timestamp = 0
        force = force or clean
        if os.path.exists(timestamp_ois_path):
            with open(timestamp_ois_path, 'r+') as f:
                ois_timestamp = f.read()
        if os.path.exists(timestamp_eleanor_path):
            with open(timestamp_eleanor_path, 'r+') as f:
                eleanor_timestamp = f.read()
        if os.path.exists(timestamp_latte_path):
            with open(timestamp_latte_path, 'r+') as f:
                latte_timestamp = f.read()
        if force or time.time() - float(ois_timestamp) > 3600 * 24 * 7:
            print("------------------ Reloading TOIs ------------------")
            ois_manager.update_tic_csvs()
            print("------------------ Reloading KOIs ------------------")
            ois_manager.update_kic_csvs()
            print("------------------ Reloading EPICs ------------------")
            ois_manager.update_epic_csvs()
            with open(os.path.join(os.path.expanduser('~'), '.sherlockpipe/timestamp_ois.txt'), 'w+') as f:
                f.write(str(time.time()))
        if (force or time.time() - float(eleanor_timestamp) > 3600 * 24 * 7) and not ois:
            print("------------------ Reloading ELEANOR TESS FFI data ------------------")
            eleanorpath = os.path.join(os.path.expanduser('~'), '.eleanor')
            eleanormetadata = eleanorpath + "/metadata"
            if clean and os.path.exists(eleanorpath) and os.path.exists(eleanormetadata):
                shutil.rmtree(eleanormetadata, ignore_errors=True)
            if not os.path.exists(eleanorpath):
                os.mkdir(eleanorpath)
            if not os.path.exists(eleanormetadata):
                os.mkdir(eleanormetadata)
            for sector in range(1, 52):
                sectorpath = eleanorpath + '/metadata/s{:04d}'.format(sector)
                if os.path.exists(sectorpath) and os.path.isdir(sectorpath) and not os.listdir(sectorpath):
                    os.rmdir(sectorpath)
                if not os.path.exists(sectorpath) or not os.path.isdir(sectorpath) or not os.listdir(sectorpath):
                    try:
                        eleanor.Update(sector)
                    except:
                        os.rmdir(sectorpath)
                        break
            with open(os.path.join(os.path.expanduser('~'), '.sherlockpipe/timestamp_eleanor.txt'), 'w+') as f:
                f.write(str(time.time()))
        if (force or time.time() - float(latte_timestamp) > 3600 * 24 * 7) and not ois:
            print("------------------ Reloading LATTE data ------------------")
            Vetter(None).update()
            with open(os.path.join(os.path.expanduser('~'), '.sherlockpipe/timestamp_latte.txt'), 'w+') as f:
                f.write(str(time.time()))
        print("DONE")


if __name__ == '__main__':
    ap = ArgumentParser(description='Updater of SHERLOCK PIPEline metadata')
    ap.add_argument('--clean', dest='clean', action='store_true', help="Whether to remove all data and download it again.")
    ap.add_argument('--only_ois', dest='ois', action='store_true', help="Whether to only refresh objects of interest.")
    ap.add_argument('--force', dest='force', action='store_true', help="Whether to ignore update timestamps and do everything again.")
    args = ap.parse_args()
    Updater().update(args.clean, args.ois, args.force)



###
### The SHERLOCK PIPEline; Searching for Hints of Exoplanets fRom Lightcurves Of spaCe-based seeKers 
### 
### Pipeline to download, detrend and search for transits in 2-min cadence TESS data
###
### Pozuelos, Thuillier & Garcia 
### V13 10/06/2020
###
### Change Log:
###               sherlock_m is devoted to find transit in lc which are given as modeled.csv(time, flux,flux_err).
###               This version chooses the signal with lartest SNR to keep searching. The SDE and FAP are printed but   
###               are not used to chose which is the best signal. This is motivated by a bug discovered by Max 
###               It has been reported to TLS developers, it will be fix soon. 
###
###

Version = 13

import os, sys
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.gridspec as gridspec
from wotan import flatten
from wotan import t14
import warnings
from astropy.stats import sigma_clip
from transitleastsquares import transitleastsquares
from transitleastsquares import transit_mask
from transitleastsquares import catalog_info
from transitleastsquares import cleaned_array
import lightkurve as lk
from lightkurve import search_lightcurvefile
from scipy import stats
import csv
import pandas as pd


# -------------------------------------------------------------------------------------------------------------------------------------------------------------
# User definitions
# -------------------------------------------------------------------------------------------------------------------------------------------------------------

TIC_ID = 136916387    # TIC_ID of our candidate
det_met = 1          # detrended method: =1 for bi-weigth ; =2 for GP matern 3/2
N_detrends = 12.      # Num. of detrend models applied
time_units = 1       # units of time: =0 (julian day) ; =1 (barycenter tess julian day)
P_protec = 10        # We procted the transit for a hypothetical Earth-size planet of Period = P_protec days. By default set = 1
Pmin = 0.5           # Minimun period in units of day
Pmax = 20            # Maximum period in units of day
minutes = 10         # time in minutes to bin the lightcurve
SNR_min = 5          # Threshold limit to keep searching. Strong restriction =>10
SDE_min = 5          # Threshold limit to keep searching. Strong restriction =>15
FAP_max = 1e-1       # Threshold limit to keep searching. Strong restriction <=1e-4
mask = 0             # = 1 means you need a mask; =0 means you do not want to use a mask 


# No need to change anything else from this point to down.


# *************************************************************************************************************************************************************
# Function that prints texts in a .log file
# *************************************************************************************************************************************************************
def logprint(*text):
#    print(*text)
    original = sys.stdout
    with open( os.path.join('TIC'+str(TIC_ID)+'_report.log'), 'a' ) as f:
        sys.stdout = f
        print(*text)
    sys.stdout = original


# *************************************************************************************************************************************************************
# Function that trim the axis (remove 'N' points)
# *************************************************************************************************************************************************************
def trim_axs(axs, N):
    axs = axs.flat
    for ax in axs[N:]:
        ax.remove()
    return axs[:N]


# *************************************************************************************************************************************************************
# Function that compute the data
# *************************************************************************************************************************************************************
def analyse(det_met, time_i, flatten_i, ab, mass, mass_min, mass_max, radius, radius_min, radius_max, id_run) :
    # ---------------------------------------------------------------------------------------------------------------------------------------------------------
    # Detrending of the data using WOTAN
    # ---------------------------------------------------------------------------------------------------------------------------------------------------------
    # By default is used a 'biweight' method.
    # It will detrend the lightcurve 'N_detrends' independent times
    from wotan import flatten

    if det_met == 2 :
        wl_min = 1
        wl_max = 12
    else :
        tdur = t14(R_s=radius, M_s=mass, P=P_protec, small_planet=True) # we define the typical duration of a small planet in this star
        wl_min = 3*tdur # minimum transit duration
        wl_max = 20*tdur # maximum transit duration

    wl_step = (wl_max-wl_min)/N_detrends
    wl = np.arange(wl_min,wl_max,wl_step) # we define all the posibles window_length that we apply
    global_flatten_lc = np.zeros((len(wl),len(flatten_i)))
    global_trend_lc = np.zeros((len(wl),len(flatten_i)))

    for i in range(0,len(wl)):
        if det_met == 2 :
            flatten_lc, trend_lc  = flatten(time_i, flatten_i, method='gp', kernel='matern', kernel_size=wl[i], return_trend=True, break_tolerance=0.5)
        else :
            flatten_lc, trend_lc  = flatten(time_i, flatten_i, window_length=wl[i], return_trend=True, method='biweight', break_tolerance=0.5 )
        global_flatten_lc[i] = flatten_lc
        global_trend_lc[i] = trend_lc

    ## save in the log file all the information concerning the detrendins applied
    warnings.filterwarnings("ignore")

    global_final = np.zeros((len(wl),len(flatten_i)))

    logprint('\n MODELS IN THE DETRENDING - Run '+str(id_run))
    logprint('========================================\n')

    if det_met == 2 :
        det_model = 'kernel_size:'
    else :
        det_model = 'window_size:'

    bins=len(time_i)*2/minutes
    bin_means, bin_edges, binnumber = stats.binned_statistic(time_i, flatten_i, statistic='mean', bins=bins)
    bin_stds, _, _ = stats.binned_statistic(time_i, flatten_i, statistic='std', bins=bins)
    bin_width = (bin_edges[1] - bin_edges[0])
    bin_centers = bin_edges[1:] - bin_width/2
    
    logprint('PDCSAP_FLUX_'+str(id_run),'\t ',' ------ ' , '\t', '------', '\t', '\t','RMS (ppm):', np.std(flatten_i)*1e6, '\t','RMS_10min (ppm):', np.std(bin_means[~np.isnan(bin_means)])*1e6 )
    for i in range(len(wl)):
        flatten = sigma_clip(global_flatten_lc[i], sigma_lower=20, sigma_upper=3)
        global_final[i]=flatten
        bins_i=len(time_i)*2/minutes
        bin_means_i, bin_edges_i, binnumber_i = stats.binned_statistic(time_i, flatten, statistic='mean', bins=bins_i)
        bin_stds_i, _, _ = stats.binned_statistic(time_i, flatten, statistic='std', bins=bins_i)
        bin_width_i = (bin_edges_i[1] - bin_edges_i[0])
        bin_centers_i = bin_edges_i[1:] - bin_width_i/2
        logprint('flatten_lc%s' %i ,'\t ','trend_lc%s' %i, '\t', det_model, format(wl[i],'0.4f'), '\t', 'RMS (ppm):', np.std(flatten)*1e6,'\t', 'RMS_10min (ppm):', np.std(bin_means_i[~np.isnan(bin_means_i)])*1e6 )

    ## save in a plot all the detrendings and all the data to inspect visually.
    cases = np.zeros((len(wl),1))
    for i in range(len(wl)):
        cases[i] = wl[i]

    figsize = (8, 8) #x,y
    cols = 3
    rows = len(cases) // cols

    shift = 2*(1.0-(np.min(flatten_i))) #shift in the between the raw and detrended data
    ylim_max = 1.0+3*(np.max(flatten_i)-1.0) #shift in the between the raw and detrended data
    ylim_min = 1.0-2.0*shift #shift in the between the raw and detrended data

    fig1, axs = plt.subplots(rows, cols, figsize=figsize, constrained_layout=True)
    axs = trim_axs(axs, len(cases))
    for ax, case in zip(axs, cases):
        if det_met == 2 :
            ax.set_title('ks=%s' % str(np.around(case, decimals=4)))
        else:
            ax.set_title('ws=%s' % str(np.around(case, decimals=4)))

        bins_i=len(time_i)*2/minutes
        bin_means_i, bin_edges_i, binnumber_i = stats.binned_statistic(time_i, global_final[np.nonzero(cases == case)[0][0]], statistic='mean', bins=bins_i)
        bin_stds_i, _, _ = stats.binned_statistic(time_i, global_final[np.nonzero(cases == case)[0][0]], statistic='std', bins=bins_i)
        bin_width_i = (bin_edges_i[1] - bin_edges_i[0])
        bin_centers_i = bin_edges_i[1:] - bin_width_i/2
        
        ax.plot(time_i, flatten_i, linewidth=0.05 ,color='black', alpha=0.75, rasterized=True)
        ax.plot(time_i, global_trend_lc[np.nonzero(cases == case)[0][0]], linewidth=1, color='orange', alpha=1.0)
        ax.plot(time_i, global_final[np.nonzero(cases == case)[0][0]]-shift, linewidth=0.05 ,color='teal', alpha=0.75, rasterized=True)
        ax.plot(bin_centers_i, bin_means_i-shift,  marker='.', markersize=2, color='firebrick', alpha=0.5, linestyle='none', rasterized=True)
        
        ax.set_ylim(ylim_min, ylim_max)
        plt.savefig('Detrends_'+'run_'+str(id_run)+'_TIC'+str(TIC_ID)+'.png', dpi=200)

    # ---------------------------------------------------------------------------------------------------------------------------------------------------------
    # Search of signals in the raw data
    # ---------------------------------------------------------------------------------------------------------------------------------------------------------
    logprint('\n SEARCH OF SIGNALS - Run', id_run)
    logprint('=================================\n')

    logprint('PDCSAP_FLUX_'+str(id_run), '\t', 'Period' ,'\t', 'Per_err', '\t', 'N.Tran', '\t', 'Mean Depth(ppt)', '\t', 'T. dur(min)', '\t','T0', '\t', 'SNR', '\t', 'SDE','\t', 'FAP\n')
    model = transitleastsquares(time_i, flatten_i)
    results_pdcsap = model.power(u=ab, M_star=mass, M_star_min=mass_min, M_star_max=mass_max, R_star=radius, R_star_min=radius_min, R_star_max=radius_max, period_min=Pmin, period_max=Pmax, n_transits_min=n_tra, show_progress_bar=False)
    x=[]

    if results_pdcsap.T0 != 0:
        for j in range(0,len(results_pdcsap.transit_depths)):
            # print (results.transit_depths[i])
            x = np.append(x, results_pdcsap.transit_depths[j])
            x = x[~np.isnan(x)]
            depth = (1.-np.mean(x))*100/0.1 #we change to ppt units
    else:
        depth = results_pdcsap.transit_depths

    logprint('-----', '\t ',
        format(results_pdcsap.period, '.5f'),'\t ',
        format(results_pdcsap.period_uncertainty, '.6f'),'\t ',
        results_pdcsap.distinct_transit_count,'\t',
        format(depth,'.3f'), '\t',
        format(results_pdcsap.duration*24*60, '.1f'),'\t',
        results_pdcsap.T0,'\t ',
        format(results_pdcsap.snr,'.3f'),'\t ',
        format(results_pdcsap.SDE,'.3f'), '\t ',
        results_pdcsap.FAP)
    logprint('\n')

    # ---------------------------------------------------------------------------------------------------------------------------------------------------------
    # Saving a plot of the best signal from raw data
    # ---------------------------------------------------------------------------------------------------------------------------------------------------------
    fig = save_plot(time_i, global_final, results_pdcsap, i)

    fig.suptitle('PDCSAP_FLUX_'+str(id_run)+' ## SNR:'+
        str(format(results_pdcsap.snr,'.3f'))+' ## SDE:'+
        str(format(results_pdcsap.SDE,'.3f'))+' ## FAP:'+
        str(results_pdcsap.FAP))
    plt.savefig('Run_'+str(id_run)+'_PDCSAP-FLUX_'+'TIC'+str(TIC_ID)+'.png', bbox_inches='tight', dpi=200)

    # ---------------------------------------------------------------------------------------------------------------------------------------------------------
    # Search of signals in the detrended data
    # ---------------------------------------------------------------------------------------------------------------------------------------------------------
    if det_met ==2 :
        logprint('Kernel_size','\t', 'Period' ,'\t', 'Per_err','\t', 'N.Tran','\t','Mean Depth(ppt)','\t','T. dur(min)','\t','T0', '\t', 'SNR', '\t', 'SDE','\t', 'FAP\n')
    else:
        logprint('Window_size','\t', 'Period' ,'\t', 'Per_err','\t', 'N.Tran','\t','Mean Depth(ppt)','\t','T. dur(min)','\t','T0', '\t', 'SNR', '\t', 'SDE','\t', 'FAP\n')
    SNR = np.zeros((len(wl), 1))
    SDE = np.zeros((len(wl), 1))
    FAP = np.zeros((len(wl), 1))
    periods = np.zeros((len(wl), 1))
    per_err = np.zeros((len(wl), 1))
    durations = np.zeros((len(wl), 1))
    tos = np.zeros((len(wl), 1))

    for i in range(len(wl)):
        print(i)
        model = transitleastsquares(time_i, global_final[i])
        results = model.power(u=ab, M_star=mass, M_star_min=mass_min, M_star_max=mass_max, R_star=radius, R_star_min=radius_min, R_star_max=radius_max, period_min=Pmin, period_max=Pmax, n_transits_min=n_tra, show_progress_bar=False)
        SNR[i] = results.snr
        SDE[i] = results.SDE
        FAP[i] = results.FAP
        periods[i] = results.period
        per_err[i] = results.period_uncertainty
        durations[i] = results.duration
        tos[i] = results.T0
        x = []
        if results.T0 != 0:
            for j in range(0, len(results.transit_depths)):
                # print (results.transit_depths[i])
                x=np.append(x, results.transit_depths[j])
                x = x[~np.isnan(x)]
                depth = (1.-np.mean(x))*100/0.1 #we change to ppt units
        else:
            depth = results.transit_depths
        logprint(format(wl[i], '.4f'), '\t ',
            format(results.period, '.5f'),'\t ',
            format(results.period_uncertainty, '.6f'),'\t ',
            results.distinct_transit_count,'\t',
            format(depth,'.3f'), '\t',
            format(results.duration*24*60, '.1f'),'\t',
            results.T0,'\t ',
            format(results.snr,'.3f'),'\t ',
            format(results.SDE,'.3f'), '\t ',
            results.FAP)

        # -----------------------------------------------------------------------------------------------------------------------------------------------------
        # Saving a plot of the best signal from detrended data
        # -----------------------------------------------------------------------------------------------------------------------------------------------------
        fig = save_plot(time_i, global_final, results, i)

        if det_met ==2 :
            fig.suptitle('Run '+str(id_run)+' ## kernel_size:'+str(format(wl[i], '.4f'))+' ## SNR:'+str(format(results.snr,'.3f'))+' ## SDE:'+str(format(results.SDE,'.3f'))+' ## FAP:'+str(results.FAP))
            #plt.savefig('TIC'+str(TIC_ID)+'_ks='+str(format(wl[i],'.4f'))+'_run_'+str(id_run)+'.png', bbox_inches='tight', dpi=200)
            plt.savefig('Run_'+str(id_run)+'_ks='+str(format(wl[i],'.4f'))+'_TIC'+str(TIC_ID)+'.png', bbox_inches='tight', dpi=200)
        else:
            fig.suptitle('Run '+str(id_run)+' ## window_size:'+str(format(wl[i], '.4f'))+' ## SNR:'+str(format(results.snr,'.3f'))+' ## SDE:'+str(format(results.SDE,'.3f'))+' ## FAP:'+str(results.FAP))
            #plt.savefig('TIC'+str(TIC_ID)+'_ws='+str(format(wl[i],'.4f'))+'_run_'+str(id_run)+'.png', bbox_inches='tight', dpi=200)
            plt.savefig('Run_'+str(id_run)+'_ws='+str(format(wl[i],'.4f'))+'_TIC'+str(TIC_ID)+'.png', bbox_inches='tight', dpi=200)
    SNR=np.nan_to_num(SNR)
    a = np.nanargmax(SNR)  #check the maximum SRN

    if results_pdcsap.snr > SNR[a]:
        logprint('\nBest Signal -->', '\t','PDCSAP_FLUX' ,'\t', 'SNR:',format(results_pdcsap.snr,'.3f'))

        if (results_pdcsap.snr > SNR_min): # and results_pdcsap.SDE > SDE_min and results_pdcsap.FAP < FAP_max):
            logprint('\nBest Signal is good enough to keep searching. Going to the next run.')
            key=1
        else:
            logprint('\nBest Signal does not look very promising. End')
            key=0
    else:
        if det_met ==2:
            logprint('\nBest Signal -->', '\t','flatten_lc'+str(a) ,'\t', 'kernel_size:', format(wl[a],'0.4f'),'\t' ,'SNR:',format(SNR[a][0],'.3f'))
        else:
            logprint('\nBest Signal -->', '\t','flatten_lc'+str(a),'\t' , 'window_size:', format(wl[a],'0.4f'),'\t' ,'SNR:',format(SNR[a][0],'.3f'))

        if (SNR[a] > SNR_min): #and SDE[a] > SDE_min and FAP[a] < FAP_max):
            logprint('\nBest Signal is good enough to keep searching. Going to the next run.')
            key=1
        else:
            logprint('\nBest Signal does not look very promising. End')
            key=0

    print("### SNR :", str(format(results.snr,'.3f')), "   SDE :", str(format(results.SDE,'.3f')), "   FAP :", str(results.FAP))

    return results_pdcsap, SNR, key, periods, durations, tos


# *************************************************************************************************************************************************************
# Function that save a plot of the data
# *************************************************************************************************************************************************************
def save_plot(time_i, global_final, results, i) :
    #start the plotting
    fig, (ax1,ax2,ax3) = plt.subplots(nrows=1, ncols=3, figsize=(10,3), constrained_layout=True)
    #1-Plot all the transits
    in_transit = transit_mask(time_i, results.period, results.duration, results.T0)
    ax1.scatter(time_i[in_transit], global_final[i][in_transit], color='red', s=2, zorder=0)
    ax1.scatter(time_i[~in_transit], global_final[i][~in_transit], color='black', alpha=0.05, s=2, zorder=0)
    ax1.plot(results.model_lightcurve_time, results.model_lightcurve_model, alpha=1, color='red', zorder=1)
    #plt.scatter(time_n, flux_new_n, color='orange', alpha=0.3, s=20, zorder=3)
    plt.xlim(time.min(), time.max())
    #plt.xlim(1362.0,1364.0)
    ax1.set(xlabel='Time (days)', ylabel='Relative flux')

    #phase folded plus binning
    ax2.plot(results.model_folded_phase, results.model_folded_model, color='red')
    ax2.scatter(results.folded_phase, results.folded_y, color='black', s=10, alpha=0.05, zorder=2)
    if (results.SDE) != 0:
        bins = 200
        bin_means, bin_edges, binnumber = stats.binned_statistic(results.folded_phase, results.folded_y, statistic='mean', bins=bins)
        bin_stds, _, _ = stats.binned_statistic(results.folded_phase, results.folded_y, statistic='std', bins=bins)
        bin_width = (bin_edges[1] - bin_edges[0])
        bin_centers = bin_edges[1:] - bin_width/2
        ax2.errorbar(bin_centers, bin_means, yerr=bin_stds/2, xerr=bin_width/2, marker='o', markersize=4, color='teal', alpha=1, linestyle='none')
        ax2.set_xlim(0.45, 0.55)
        ax2.set(xlabel='Phase', ylabel='Relative flux')
        plt.ticklabel_format(useOffset=False)
    else:
        ax2.set_xlim(0.45, 0.55)
        ax2.set(xlabel='Phase', ylabel='Relative flux')
        plt.ticklabel_format(useOffset=False)

    #SDE
    ax3 = plt.gca()
    ax3.axvline(results.period, alpha=0.4, lw=3)
    plt.xlim(np.min(results.periods), np.max(results.periods))
    for n in range(2, 10):
        ax3.axvline(n*results.period, alpha=0.4, lw=1, linestyle="dashed")
        ax3.axvline(results.period / n, alpha=0.4, lw=1, linestyle="dashed")
    ax3.set(xlabel='Period (days)', ylabel='SDE')
    ax3.plot(results.periods, results.power, color='black', lw=0.5)
    ax3.set_xlim(0., max(results.periods))

    return fig


# *************************************************************************************************************************************************************
# Function that add a mask on the raw data
# *************************************************************************************************************************************************************
def add_mask(time_i, flatten_i, results_pdcsap, periods, durations, tos, a):
    if results_pdcsap.snr > SNR[a]:
        intransit = transit_mask(time_i, results_pdcsap.period, 2*results_pdcsap.duration, results_pdcsap.T0)
    else:
        intransit = transit_mask(time_i, periods[a], 2*durations[a], tos[a])

    flatten_i[intransit]=np.nan
    time_j, flatten_j = cleaned_array(time_i, flatten_i)

    return time_j, flatten_j


# *************************************************************************************************************************************************************
# Main
# *************************************************************************************************************************************************************

# -------------------------------------------------------------------------------------------------------------------------------------------------------------
#      Initialization
# -------------------------------------------------------------------------------------------------------------------------------------------------------------

# Search of data by lightkurve function concerning the TIC_ID provided
# It retrieves all the data (sectors) available. 
# save in log file the info
ab, mass, mass_min, mass_max, radius, radius_min, radius_max = catalog_info(TIC_ID=TIC_ID)
#search and donwload the TESS data using lightkurve
lcf= lk.search_lightcurvefile('TIC '+str(TIC_ID), mission="tess").download_all()

#we set the minum number of transits based on the sectors available
if len(lcf) > 1: 
    n_tra = 2
else:
    n_tra = 1

# saving usere definitions to keep the track of what have be done already
logprint('\n SHERLOCK (Searching for Hints of Exoplanets fRom Lightcurves Of spaCe-base seeKers)')
logprint('\n Version ', Version )
logprint('\n USER DEFINITIONS')
logprint('========================\n')
logprint('TIC_ID: ', TIC_ID)

if det_met == 2 :
    logprint('Detrend method: Gaussian Process Matern 2/3')
else:
    logprint('Detrend method: Bi-Weight')

logprint('No of detrend models applied: ', N_detrends)
logprint('No of sectors available: ', len(lcf))
logprint('Minimum number of transits: ', n_tra)
logprint('Period planet protected: ', P_protec)
logprint('Minimum Period (d): ', Pmin)
logprint('Maximum Period (d): ', Pmax)
logprint('Binning size (min): ', minutes)

if time_units ==1 :
    logprint('Units of time: Barycenter TESS Julian Day')
else:
    logprint('Units of time: Julian Day')
if mask ==1 :
    logprint('Mask: yes')
else:
    logprint('Mask: no')

logprint('Threshold limit for SNR: ', SNR_min)
logprint('Threshold limit for SDE: ', SDE_min)
logprint('Threshold limit for FAP: ', FAP_max)

# We save here in a log file all the parameters available of the star.

mass_min = mass-mass_min
mass_max = mass+mass_max
radius_min = radius-radius_min
radius_max = radius+radius_max

logprint('\n STELLAR PROPERTIES FOR THE SIGNAL SEARCH')
logprint('================================================\n')
logprint('limb-darkening estimates using quadratic LD (a,b)=', ab)
logprint('mass =', format(mass,'0.5f'))
logprint('mass_min =', format(mass_min,'0.5f'))
logprint('mass_max =', format(mass_max,'0.5f'))
logprint('radius =', format(radius,'0.5f'))
logprint('radius_min =', format(radius_min,'0.5f'))
logprint('radius_max =', format(radius_max,'0.5f'))



logprint('\n SECTORS INFO')
logprint('================================================\n')
logprint('Sectors :', lcf)

lc=lcf.PDCSAP_FLUX.stitch().remove_nans() # remove of the nans
flux=lc.flux
flux_err=lc.flux_err
if time_units == 0:
    time=lc.astropy_time.jd #transform from TESS julian date to jd
elif time_units == 1:
    time=lc.time  #keep the TESS julian date

#we insert here the lc as modeled.csv
df = pd.read_csv('modeled.csv',float_precision='round_trip', sep=',',
                 usecols=['#TBJD','flux', 'flux_err'])

lc_new=lk.LightCurve(time=df['#TBJD'], flux=df['flux'],flux_err=df['flux_err'])
#lc_new=lk.LightCurve(time=time, flux=flux,flux_err=flux_err)
flux_clean=lc_new.remove_outliers(sigma_lower=float('inf'), sigma_upper=3) #remove outliers over 3sigma

if mask==1:
    #first mask (example of how to mask a range)
    time_in=1449.8
    time_out=1451.50
    mask1 = (flux_clean.time < time_in) | (flux_clean.time > time_out) 
    flux_clean = flux_clean[mask1]
    time_b=1463.5
    mask2 = (flux_clean.time < time_b)
    flux_clean = flux_clean[mask2]
    logprint('** Masking the lightcurve **')
else:
    None
        

# -------------------------------------------------------------------------------------------------------------------------------------------------------------
#      Start of the runs
# -------------------------------------------------------------------------------------------------------------------------------------------------------------
print("________________________________ run 1 ________________________________")
id_run = 1
time_i = flux_clean.time
flatten_i = flux_clean.flux
results_pdcsap, SNR, key, periods, durations, tos = analyse(det_met, time_i, flatten_i, ab, mass, mass_min, mass_max, radius, radius_min, radius_max, id_run)

while key == 1:
    id_run += 1
    print("________________________________ run", id_run, "________________________________")
    SNR=np.nan_to_num(SNR)
    a = np.nanargmax(SNR)  # check the maximum SDE

    # applying a new mask
    time_i, flatten_i = add_mask(time_i, flatten_i, results_pdcsap, periods, durations, tos, a)

    # analyzing the data with the new mask
    results_pdcsap, SNR, key, periods, durations, tos = analyse(det_met, time_i, flatten_i, ab, mass, mass_min, mass_max, radius, radius_min, radius_max, id_run)


# end



import numpy as np
import time
import matplotlib.pyplot as plt
import os

#
# Script para calcular las HZ basado en las ecuaciones de 
# kopparapu et al (2013)
#
# Estas ecuaciones son validas para estrellas entre 2600<Teff<7200k
# Hay que dar la Teff y la L de la estrella en cuestion
#


Teff = 2983. #temperatura effectiva de la estrella que estemos considerando
L = 0.001 #luminosidad en L_sun
L_sun = 1.
Ts = Teff-5780.

#ecuaciones (2) y (3) en kopparapu et al 2013, para orbitas no eccentricas
def Seff(Seff_sun,a,b,c,d):
    Seff = Seff_sun + a*Ts + b*Ts**2 + c*Ts**3 + d*Ts**4
    return Seff

def Dis(Seff):
    Dis = (L/L_sun/Seff)**0.5
    return Dis

#considerando ecuaciones (3) y (4) para orbitas eccentricas en kopparapu et al 2013
def Seff_prime(A,e):
    Seff_prime=A/np.sqrt(1.-e**2)
    return Seff_prime

##
## coeficientes para calcular los flujos estelares (tabla 3 en kopparapu et al 2013)
##

e_end = 1.00
Nout = 10000
e = np.linspace(0.0001,e_end,Nout,endpoint=False)

#Recent Venus

s_eff_rv = 1.7763
a_rv = 1.4335e-4
b_rv = 3.3954e-9
c_rv = -7.6364e-12
d_rv = -1.1950e-15
Seff_rv=Seff(s_eff_rv,a_rv,b_rv,c_rv,d_rv)
Dis_rv=Dis(Seff_rv)
Seff_prime_rv=Seff_prime(Seff_rv,e)
Dis_prime_rv=Dis(Seff_prime_rv)
print("Recent Venus distance: ", Dis_rv, "AU")


#Runaway Greenhouse

s_eff_rg = 1.0385
a_rg = 1.2456e-4
b_rg = 1.4612e-8
c_rg = -7.6345e-12
d_rg = -1.7511e-15
Seff_rg=Seff(s_eff_rg,a_rg,b_rg,c_rg,d_rg)
Dis_rg=Dis(Seff_rg)
Seff_prime_rg=Seff_prime(Seff_rg,e)
Dis_prime_rg=Dis(Seff_prime_rg)
print("Runaway Greenhouse distance: ", Dis_rg, "AU")

#Moist Greenhouse

s_eff_mog = 1.0146
a_mog = 8.1884e-5
b_mog = 1.9394e-9
c_mog = -4.3618e-12
d_mog = -6.8260e-16
Seff_mog=Seff(s_eff_mog,a_mog,b_mog,c_mog,d_mog)
Dis_mog=Dis(Seff_mog)
Seff_prime_mog=Seff_prime(Seff_mog,e)
Dis_prime_mog=Dis(Seff_prime_mog)
print("Moist Greenhouse distance: ", Dis_mog, "AU")

#Maximun Greenhouse

s_eff_mag = 0.3507
a_mag = 5.9578e-5
b_mag = 1.6707e-9
c_mag = -3.0058e-12
d_mag = -5.1925e-16
Seff_mag=Seff(s_eff_mag,a_mag,b_mag,c_mag,d_mag)
Dis_mag=Dis(Seff_mag)
Seff_prime_mag=Seff_prime(Seff_mag,e)
Dis_prime_mag=Dis(Seff_prime_mag)
print("Maximum Greenhouse distance: ", Dis_mag, "AU")

#Early Mars

s_eff_em = 0.3207
a_em = 5.4471e-5
b_em = 1.5275e-9
c_em = -2.1709e-12
d_em = -3.8282e-16
Seff_em=Seff(s_eff_em,a_em,b_em,c_em,d_em)
Dis_em=Dis(Seff_em)
Seff_prime_em=Seff_prime(Seff_em,e)
Dis_prime_em=Dis(Seff_prime_em)
print("Early Mars distance: ", Dis_em, "AU")


#
# La zona de habitabilidad esta entre Moist Greenhouse y Maximum Greenhouse
#

print("Zona de Habitabilidad - Conservative: ", Dis_mog, "-", Dis_mag, "AU")
print("Zona de Habitabilidad - Optimistic: ", Dis_rv, "-", Dis_em, "AU")


##
## Vamos a plotear eccentricidad vs semimajor axis, para considerar orbitas eccentricas
##


a_end = 10.
Nout = 10000
a = np.linspace(0.0001,a_end,Nout,endpoint=False)

fig1 = plt.figure(figsize=(5,3))
plt.subplot(111)
#plt.plot(a,1.-Dis_rv/a,'b-',a,Dis_em/a-1,'b-')
#plt.plot(a,1.-Dis_mog/a,'b-',a,Dis_mag/a-1,'b-')
#plt.plot(a,1.-Dis_prime_rv/a,'r--',a,Dis_prime_em/a-1,'r--' )
#plt.plot(a,1.-Dis_prime_mog/a,'r--',a,Dis_prime_mag/a-1,'r--' )
colorx='green'
HZ_opt=[]
a_opt=[]
for i in range(0, len(a)):
    if (Dis_em/a[i]-1. > 1.-Dis_rv/a[i] and 1.-Dis_rv/a[i] > 0.):
        HZ_opt.append(1.-Dis_rv/a[i])
        a_opt.append(a[i])
    if (Dis_em/a[i]-1. < 1.-Dis_rv/a[i] and Dis_em/a[i]-1 > 0.):
        HZ_opt.append(Dis_em/a[i]-1.)
        a_opt.append(a[i])
colorx='green'
HZ_con=[]
a_con=[]
for i in range(0, len(a)):
    if (Dis_mag/a[i]-1. > 1.-Dis_mog/a[i] and 1.-Dis_mog/a[i] > 0.):
        HZ_con.append(1.-Dis_mog/a[i])
        a_con.append(a[i])
    if (Dis_mag/a[i]-1. < 1.-Dis_mog/a[i] and Dis_mag/a[i]-1 > 0.):
        HZ_con.append(Dis_mag/a[i]-1.)
        a_con.append(a[i])


plt.fill_between(a_opt,HZ_opt, 0,facecolor=colorx, alpha=0.5,edgecolor="none")
plt.fill_between(a_con,HZ_con, 0,facecolor=colorx, alpha=0.8,edgecolor="none")
plt.text(0.015,0.35,"Optimistic HZ",size=10)
plt.plot(0.01,0.36,"s",color=colorx,alpha=0.5)
plt.text(0.015,0.32,"Conservative HZ",size=10)
plt.plot(0.01,0.33,"s",color=colorx,alpha=0.8)




##calculo del planeta a traves de su periodo orbital, especial para TESS

p1 = 15# periodo orbital en dias dado por TESS
T1 = p1*24.*3600. #periodo orbital en segundos

p2 = 18  # periodo orbital en dias dado por TESS
T2 = p2*24.*3600. #periodo orbital en segundos

#p3 = 217.2  # periodo orbital en dias dado por TESS
#T3 = p3*24.*3600. #periodo orbital en segundos

G = 6.674e-11 #m3 kg-1 s-2
Mass = 0.12 #Msun
Mass_kg = Mass*2.e30 #masa del cuerpo central en kg

semieje1 = (G*Mass_kg*T1**2/4./(np.pi**2))**(1./3.)
semieje1_au = semieje1/1.496e11
print("a1 = ", semieje1_au)


semieje2 = (G*Mass_kg*T2**2/4./(np.pi**2))**(1./3.)
semieje2_au = semieje2/1.496e11
print("a2 = ", semieje2_au)

#semieje3 = (G*Mass_kg*T3**2/4./(np.pi**2))**(1./3.)
#semieje3_au = semieje3/1.496e11
#print "a3 = ", semieje3_au

r_earth=5

plt.errorbar(semieje1_au,0.042,yerr=[[0.028],[0.040]],marker='o',markersize=1.44*r_earth,color='cyan',markeredgecolor='black',ecolor='black',elinewidth=0.75,markeredgewidth=0.75)

plt.errorbar(semieje2_au,0.034,yerr=[[0.024],[0.044]],marker='o',markersize=1*r_earth,color='cyan',markeredgecolor='black',ecolor='black',elinewidth=0.75,markeredgewidth=0.75)

#plt.errorbar(semieje3_au,0.55,0.10,marker='o',markersize=6.5*r_earth,color='cyan',markeredgecolor='black',ecolor='black',elinewidth=0.75,markeredgewidth=0.75)


#plt.text(0.018,0.017,"03",size=6)
#plt.plot(0.0316,0.02,'o',markersize=1.43*r_earth,color='Orange')
#plt.text(0.0316,0.02,"02",size=6)
#plt.plot(0.0505,0.02,'o',markersize=1.34*r_earth,color='Blue')
#plt.text(0.0505,0.02,"01",size=6)



#plt.fill_between(a,Dis_em/a-1., 0, where=Dis_em/a-1. < 1.-Dis_rv/a, facecolor=colorx, alpha=0.5,edgecolor="none")

plt.ylabel("Eccentricity",size=12)
plt.xlabel("Semimajor axis (au)",size=12)
plt.ylim(0,0.5)
plt.xlim(0,0.075)
#plt.title("GJ 628")
#####
#### Formato de salida: png, jpg, png o eps
####

formato = str('.png')
#formato = str('.png')
#formato = str('.jpg')
#formato = str('.eps')
plt.savefig('sp0004-1709'+formato,
                  bbox_inches='tight', # Elimina margenes en blanco
                  dpi=200)




# -----------------------------
'''

This test case is carried out with a pre-downloaded lighcurve file from TIC 55525572 Sector 5.

Testing whether the plots are produced as expected. 

python -m unittest tests/test_plotting.py
'''

import os
import sys
import time 
import datetime
import unittest
from dateutil import parser


import warnings
warnings.filterwarnings("ignore")


import LATTE.LATTEutils as utils

# test the downloading of the data
# get the outir (or indir) path
syspath = str(os.path.abspath(utils.__file__))[0:-14]

indir = "./LATTE/test_output"		


# -------------
# test with these input parameters (this data is already downloaded so it's only testing that the plotting part works)
tic = '55525572'
sector = '5'
sectors = [5]
transit_list = [1454.7]
transit_sec = '5'
# -------------

# create a mock 'argparser' becasuse the argparse function doesn't work within the unittest module 
# for now use all of the default values. 
class Namespace:
	def __init__(self, **kwargs):  # arbitrary number of input values
		self.__dict__.update(kwargs)

global args
args = Namespace(new_data=False, tic='no',sector='no', targetlist='no', 
	noshow=True, o=False, auto=False,nickname='noname', FFI=False, save=True,north=False,new_path=False,mpi=False)

# test it with the default arguments (except noshow - we don't want the plots to show up in the test run)
# --------------

class TestDataPlotting(unittest.TestCase):
	
	def test_plot(self):

		# download data needed to generate the plots
		alltime, allflux, allflux_err, all_md, alltimebinned, allfluxbinned, allx1, allx2, ally1, ally2, alltime12, allfbkg, start_sec, end_sec, in_sec, tessmag, teff, srad = utils.download_data(indir,sector, tic, binfac = 5, test = './LATTE/tests/tic55525572_lc.fits')
		X1_list, X4_list, oot_list, intr_list, bkg_list, apmask_list, arrshape_list, t_list, T0_list, tpf_filt_list = utils.download_tpf_mast(indir, transit_sec, transit_list, tic, test = './LATTE/tests/tic55525572_tp.fits')
		
		# this function downloads the data and saved plots that show the aperture sizes (for the large and small apertures)
		TESS_unbinned_t_l, TESS_binned_t_l, small_binned_t_l, TESS_unbinned_l, TESS_binned_l, small_binned_l, tpf_list = utils.download_tpf_lightkurve(indir, transit_list, sectors, tic, test = './LATTE/tests/tic55525572_tp.fits')
		# ---------

		# create the plots that the normal routine makes
		utils.plot_full_md(tic, indir, alltime, allflux, all_md, alltimebinned, allfluxbinned, transit_list, args)
		utils.plot_centroid(tic, indir, alltime12, allx1, ally1, allx2, ally2, transit_list, args)
		utils.plot_background(tic, indir, alltime, allfbkg, transit_list, args)
		utils.plot_pixel_level_LC(tic, indir, X1_list, X4_list, oot_list, intr_list, bkg_list, apmask_list, arrshape_list, t_list, transit_list, args)
		utils.plot_aperturesize(tic, indir, TESS_unbinned_t_l, TESS_binned_t_l, small_binned_t_l, TESS_unbinned_l, TESS_binned_l, small_binned_l, transit_list, args)

		# --------
		# now check that the plots were actually made!

		# in order to be able to run this code multiple times, assert that the files were created in the last minuet - if the code takes lomger than that to run something is goimg wrong...?
		# without checking when the file is made, the files would have to be manually deleted after every test

		# these are the paths of the files that should have bee created
		full_LC_path = '{}/55525572/55525572_fullLC_md.png'.format(indir)
		full_centroid = '{}/55525572/55525572_centroids.png'.format(indir)
		bkg_path = '{}/55525572/55525572_background.png'.format(indir)
		pixel_path = '{}/55525572/55525572_individual_pixel_LCs_0.png'.format(indir)

		ap_LC_path = '{}/55525572/55525572_aperture_size.png'.format(indir)
		apertures_path = '{}/55525572/55525572_apertures_0.png'.format(indir)

		# Get file's Last modification time stamp only in terms of seconds since epoch 
		time_created_full_LC= os.path.getmtime(full_LC_path)
		time_created_centroid = os.path.getmtime(full_centroid)
		time_created_bkg = os.path.getmtime(bkg_path)
		time_created_pixel = os.path.getmtime(pixel_path)
		 
		time_created_ap_LC = os.path.getmtime(ap_LC_path)
		time_created_apertures = os.path.getmtime(apertures_path)

		# Convert seconds since epoch to readable timestamp
		t_create_full_LC = parser.parse(time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time_created_full_LC)))
		t_create_centroid = parser.parse(time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time_created_centroid)))		
		t_create_bkg = parser.parse(time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time_created_bkg)))
		t_create_pixel = parser.parse(time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time_created_pixel)))
		
		t_create_ap_LC = parser.parse(time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time_created_ap_LC)))
		t_create_apertures = parser.parse(time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time_created_apertures)))
				
		# the time now (needed to get the time since creation)
		t_now = (datetime.datetime.now())

		# -------
		# time difference in minutes
		time_since_creation_full_LC =  ((t_now - t_create_full_LC).seconds / 60)
		time_since_creation_centroid =  ((t_now - t_create_centroid).seconds / 60) 
		time_since_creation_bkg =  ((t_now - t_create_bkg).seconds / 60) 
		time_since_creation_pixel =  ((t_now - t_create_pixel).seconds / 60) 

		time_since_creation_bkg =  ((t_now - t_create_ap_LC).seconds / 60) 
		time_since_creation_pixel =  ((t_now - t_create_apertures).seconds / 60) 

		# check that the expected files were created less than a minute ago
		self.assertLess(time_since_creation_full_LC, 1, "No (new) full LC plot was made in the last five minutes") # a less than b
		self.assertLess(time_since_creation_centroid, 1, "No (new) centroid plot was made in the last five minutes") # a less than b
		self.assertLess(time_since_creation_bkg, 1, "No (new) background plot was made in the last five minutes") # a less than b
		self.assertLess(time_since_creation_pixel, 1, "No (new) pixel level LC plot was made in the last five minutes") # a less than b

		self.assertLess(time_since_creation_bkg, 1, "No (new) background plot was made in the last five minutes") # a less than b
		self.assertLess(time_since_creation_pixel, 1, "No (new) pixel level LC plot was made in the last five minutes") # a less than b

if __name__ == '__main__':

	unittest.main()







'''

This script is designed to test the LATTE python code to ensure that the server response is working. 

NOTE: requires connection to the internet and to the archive database. If these tests fail, first check that these connections 
are working by accessing the website: https://archive.stsci.edu/missions/tess/download_scripts/ in a browser.

'''
import os
import unittest
import requests

import warnings
warnings.filterwarnings("ignore")

import sys
import LATTE.LATTEutils as utils

# test the downloading of the data
# get the outir (or indir) path
syspath = str(os.path.abspath(utils.__file__))[0:-14]

with open("{}/_config.txt".format(syspath), 'r') as f:
	indir = str(f.readlines()[-1])
		
def download_LC_data():
	# function to reach the external server to download the scipt - this is for the first sector only
	LC_url = "https://archive.stsci.edu/missions/tess/download_scripts/sector/tesscurl_sector_1_lc.sh"
	r_LC = requests.get(LC_url) # create HTTP response object
	
	return r_LC.status_code

def download_TP_data():
	# function to reach the external server to download the scipt - this is for the first sector only
	TP_url = "https://archive.stsci.edu/missions/tess/download_scripts/sector/tesscurl_sector_1_tp.sh"
	r_TP = requests.get(TP_url) # create HTTP response object 

	return r_TP.status_code	

def download_target_list():
	# function to reach the external server to download the scipt - this is for the first sector only
	target_list = "https://tess.mit.edu/wp-content/uploads/all_targets_S001_v1.txt"
	r_target_list = requests.get(target_list) # create HTTP response object

	return r_target_list.status_code	


class TestServerResponse(unittest.TestCase):
	
	def test_LC_request_response(self):
		# Call the service, which will send a request to the server.
		responseLC = download_LC_data()
		responseTP = download_TP_data()
		responseTL = download_target_list()

		# If the request is sent successfully, then I expect a response to be returned.
		self.assertEqual(responseLC, 200, "LC data Download link does not work - can't connect to the server")
		self.assertEqual(responseTP, 200, "TP data Download link does not work - can't connect to the server")
		self.assertEqual(responseTL, 200, "Target list data Download link does not work - can't connect to the server")


if __name__ == '__main__':

	unittest.main()

